from django.test import TestCase

from dcim.models import Device, Interface
from ipam.models import VRF
from utilities.testing import create_test_device

from netbox_routing.forms import *
from netbox_routing.models import *

__all__ = (
    'OSPFInstanceTestCase',
    'OSPFAreaTestCase',
    'OSPFInterfaceTestCase',
)


class OSPFInstanceTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.vrf = VRF.objects.create(name='Test VRF')
        cls.device = create_test_device(name='Device 1')

    def test_instance(self):
        form = OSPFInstanceForm(
            data={
                'name': 'Instance 1',
                'process_id': '0',
                'router_id': '10.10.10.1',
                'device': Device.objects.first().pk,
            }
        )
        if not form.is_valid():
            print(form.errors)
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def test_instance_with_vrf(self):
        form = OSPFInstanceForm(
            data={
                'name': 'Instance 2',
                'process_id': '1',
                'router_id': '20.20.20.1',
                'device': Device.objects.first().pk,
                'vrf': VRF.objects.first().pk,
            }
        )
        if not form.is_valid():
            print(form.errors)
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())


class OSPFAreaTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        pass

    def test_valid_area_id_with_ip(self):
        form = OSPFAreaForm(
            data={
                'area_id': '0.0.0.0',
                'area_type': 'standard',
            }
        )
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def test_valid_area_id_with_integer(self):
        form = OSPFAreaForm(
            data={
                'area_id': '0',
                'area_type': 'standard',
            }
        )
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def test_invalid_area(self):
        form = OSPFAreaForm(
            data={
                'area_id': 'a.a.a.a',
                'area_type': 'standard',
            }
        )
        self.assertFalse(form.is_valid())
        with self.assertRaises(ValueError):
            form.save()


class OSPFInterfaceTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        device = create_test_device(name='Device 1')
        cls.interface = Interface.objects.create(
            name='Interface 1', device=device, type='virtual'
        )
        cls.instance = OSPFInstance.objects.create(
            name='Instance 1', router_id='10.10.10.1', process_id=0, device_id=device.pk
        )
        cls.area = OSPFArea.objects.create(area_id='0.0.0.0')

    def test_interface_with_correct_device(self):
        form = OSPFInterfaceForm(
            data={
                'device': Device.objects.first().pk,
                'interface': Interface.objects.first().pk,
                'instance': OSPFInstance.objects.first().pk,
                'area': OSPFArea.objects.first().pk,
                'passive': True,
            }
        )
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def test_interface_with_incorrect_device(self):
        device = create_test_device(name='Device 2')
        interface = Interface.objects.create(
            name='Interface 1', device=device, type='virtual'
        )

        form = OSPFInterfaceForm(
            data={
                'device': device.pk,
                'interface': interface.pk,
                'instance': OSPFInstance.objects.first().pk,
                'area': OSPFArea.objects.first().pk,
            }
        )
        self.assertFalse(form.is_valid())
        with self.assertRaises(ValueError):
            form.save()



from django.test import TestCase

from ipam.models import Prefix
from netbox_routing.models.objects import *
from netbox_routing.models.community import *

from netbox_routing.choices import ActionChoices
from netbox_routing.forms.objects import *

__all__ = (
    'ASPathTestCase',
    'ASPathEntryTestCase',
    'PrefixListTestCase',
    'PrefixListEntryTestCase',
    'RouteMapTestCase',
    'RouteMapEntryTestCase',
)


class ASPathTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        pass

    def test_object(self):
        form = ASPathForm(
            data={
                'name': 'AS Path List 1',
                'description': 'Description',
                'comment': 'Comment',
            }
        )
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def test_minimal_object(self):
        form = ASPathForm(
            data={
                'name': 'AS Path List 1',
            }
        )
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def test_form_invalid(self):
        form = ASPathForm(
            data={
                'name': 'AS Path List 1 Too Long Name Needs to be over 100 characters long and enough characters to cau'
                'se an error with the form validation',
            }
        )
        self.assertFalse(form.is_valid())
        with self.assertRaises(ValueError):
            form.save()


class ASPathEntryTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.aspath = ASPath.objects.create(
            name='AS Path List 1',
        )

    def test_object(self):
        form = ASPathEntryForm(
            data={
                'aspath': self.aspath,
                'sequence': 1,
                'action': ActionChoices.PERMIT,
                'pattern': '^2242$',
            }
        )
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def test_form_invalid(self):
        form = ASPathEntryForm(data={'aspath': self.aspath, 'pattern': '^2242$'})
        self.assertFalse(form.is_valid())
        with self.assertRaises(ValueError):
            form.save()


class PrefixListTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        pass

    def test_object(self):
        form = PrefixListForm(
            data={
                'name': 'Prefix List 1',
                'family': 4,
                'description': 'Description',
                'comment': 'Comment',
            }
        )
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def test_minimal_object(self):
        form = PrefixListForm(
            data={
                'name': 'Prefix List 1',
                'family': 6,
            }
        )
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def test_form_invalid(self):
        form = PrefixListForm(
            data={
                'name': 'Prefix List 1',
            }
        )
        self.assertFalse(form.is_valid())
        with self.assertRaises(ValueError):
            form.save()


class PrefixListEntryTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.prefix_list = PrefixList.objects.create(
            name='Prefix List 1',
            family=4,
        )
        cls.prefix = Prefix(prefix='10.0.0.0/8')
        cls.prefix.full_clean()
        cls.prefix.save()

        cls.custom_prefix = CustomPrefix(prefix='192.168.0.0/16')
        cls.custom_prefix.full_clean()
        cls.custom_prefix.save()

    def test_prefix(self):
        form = PrefixListEntryForm(
            data={
                'prefix_list': self.prefix_list,
                'sequence': 1,
                'action': ActionChoices.PERMIT,
                'ipam_prefix': self.prefix.pk,
            }
        )
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def test_custom_prefix(self):
        form = PrefixListEntryForm(
            data={
                'prefix_list': self.prefix_list,
                'sequence': 1,
                'action': ActionChoices.PERMIT,
                'custom_prefix': self.custom_prefix.pk,
            }
        )
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def test_form_invalid(self):
        form = PrefixListEntryForm(
            data={
                'prefix_list': self.prefix_list,
                'prefix': self.prefix.pk,
                'custom_prefix': self.custom_prefix.pk,
            }
        )
        self.assertFalse(form.is_valid())
        with self.assertRaises(ValueError):
            form.save()


class RouteMapTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        pass

    def test_object(self):
        form = RouteMapForm(
            data={
                'name': 'Route Map 1',
                'description': 'Description',
                'comment': 'Comment',
            }
        )
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def test_minimal_object(self):
        form = RouteMapForm(
            data={
                'name': 'Route Map 1',
            }
        )
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def test_form_invalid(self):
        form = RouteMapForm(
            data={
                'name': 'Route Map 1 Too Long Name Needs to be over 100 characters long and enough characters to cau'
                'se an error with the form validation',
            }
        )
        self.assertFalse(form.is_valid())
        with self.assertRaises(ValueError):
            form.save()


class RouteMapEntryTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.route_map = RouteMap.objects.create(
            name='Route Map 1',
        )
        cls.community = Community.objects.create(community='64512', status='active')

    def test_object(self):
        form = RouteMapEntryForm(
            data={
                'route_map': self.route_map,
                'sequence': 1,
                'action': ActionChoices.PERMIT,
                'match_community': [
                    self.community.pk,
                ],
                'match_prefix_list': [],
                'match_community_list': [],
                'match_aspath': [],
                'match': {},
            }
        )
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())
        self.assertEqual(
            list(form.instance.match_community.all()),
            [
                self.community,
            ],
        )

    def test_form_invalid(self):
        form = RouteMapEntryForm(
            data={
                'route_map': self.route_map,
            }
        )
        self.assertFalse(form.is_valid())
        with self.assertRaises(ValueError):
            form.save()



from netbox.forms import NetBoxModelFilterSetForm
from netbox_routing.models.objects import *

__all__ = (
    'CustomPrefixFilterForm',
    'PrefixListFilterForm',
    'PrefixListEntryFilterForm',
    'RouteMapFilterForm',
    'RouteMapEntryFilterForm',
    'ASPathFilterForm',
    'ASPathEntryFilterForm',
)


class PrefixListFilterForm(NetBoxModelFilterSetForm):
    model = PrefixList


class PrefixListEntryFilterForm(NetBoxModelFilterSetForm):
    model = PrefixListEntry


class CustomPrefixFilterForm(NetBoxModelFilterSetForm):
    model = CustomPrefix


class RouteMapFilterForm(NetBoxModelFilterSetForm):
    model = RouteMap


class RouteMapEntryFilterForm(NetBoxModelFilterSetForm):
    model = RouteMapEntry


class ASPathFilterForm(NetBoxModelFilterSetForm):
    model = ASPath


class ASPathEntryFilterForm(NetBoxModelFilterSetForm):
    model = ASPathEntry



import django_tables2 as tables
from django.utils.translation import gettext as _

from netbox.tables import NetBoxTable, columns, ManyToManyColumn
from netbox_routing.models.bgp import *

__all__ = (
    'BGPSettingTable',
    'BGPRouterTable',
    'BGPScopeTable',
    'BGPAddressFamilyTable',
    'BGPPeerTable',
    'BGPPeerAddressFamilyTable',
    'BGPPeerTemplateTable',
    'BGPPolicyTemplateTable',
    'BGPSessionTemplateTable',
    'BFDProfileTable',
)

from tenancy.tables import TenancyColumnsMixin


class BGPSettingTable(NetBoxTable):
    assigned_object_type = columns.ContentTypeColumn(verbose_name=_('Object Type'))
    assigned_object = tables.Column(
        linkify=True, orderable=False, verbose_name=_('Object')
    )

    class Meta(NetBoxTable.Meta):
        model = BGPSetting
        fields = ('pk', 'id', 'assigned_object_type', 'assigned_object', 'key', 'value')
        default_columns = (
            'pk',
            'id',
            'assigned_object_type',
            'assigned_object',
            'key',
            'value',
        )


class BGPPeerTemplateTable(TenancyColumnsMixin, NetBoxTable):
    name = tables.Column(
        verbose_name=_('Name'),
        linkify=True,
    )

    class Meta(NetBoxTable.Meta):
        model = BGPPeerTemplate
        fields = (
            'pk',
            'id',
            'name',
        )
        default_columns = ('pk', 'id', 'name')


class BGPPolicyTemplateTable(TenancyColumnsMixin, NetBoxTable):
    name = tables.Column(
        verbose_name=_('Name'),
        linkify=True,
    )
    parents = ManyToManyColumn(
        verbose_name=_('Parents'),
        linkify_item=True,
    )

    class Meta(NetBoxTable.Meta):
        model = BGPPolicyTemplate
        fields = (
            'pk',
            'id',
            'name',
            'parents',
            'enabled',
            'prefixlist_out',
            'prefixlist_in',
            'routemap_out',
            'routemap_in',
            'tenant',
        )
        default_columns = ('pk', 'id', 'name')


class BGPSessionTemplateTable(TenancyColumnsMixin, NetBoxTable):
    name = tables.Column(
        verbose_name=_('Name'),
        linkify=True,
    )
    parent = tables.Column(
        verbose_name=_('Parent'),
        linkify=True,
    )
    asn = tables.Column(
        verbose_name=_('Parent'),
        linkify=True,
    )

    class Meta(NetBoxTable.Meta):
        model = BGPSessionTemplate
        fields = (
            'pk',
            'id',
            'name',
            'parent',
            'enabled',
            'local_as',
            'remote_as',
            'bfd',
            'ttl',
            'password',
            'tenant',
        )
        default_columns = ('pk', 'id', 'name')


class BGPRouterTable(TenancyColumnsMixin, NetBoxTable):
    name = tables.Column(linkify=True, verbose_name=_('Name'))
    assigned_object_type = columns.ContentTypeColumn(verbose_name=_('Object Type'))
    assigned_object = tables.Column(
        linkify=True, orderable=False, verbose_name=_('Object')
    )

    class Meta(NetBoxTable.Meta):
        model = BGPRouter
        fields = (
            'pk',
            'id',
            'name',
            'assigned_object_type',
            'assigned_object',
            'asn',
            'tenant_group',
            'tenant',
        )
        default_columns = ('pk', 'id', 'name', 'assigned_object', 'asn')


class BGPScopeTable(TenancyColumnsMixin, NetBoxTable):
    class Meta(NetBoxTable.Meta):
        model = BGPScope
        fields = (
            'pk',
            'id',
            'router',
            'vrf',
            'tenant_group',
            'tenant',
        )
        default_columns = ('pk', 'id', 'router', 'vrf')


class BGPAddressFamilyTable(TenancyColumnsMixin, NetBoxTable):
    class Meta(NetBoxTable.Meta):
        model = BGPAddressFamily
        fields = (
            'pk',
            'id',
            'scope',
            'address_family',
            'tenant_group',
            'tenant',
        )
        default_columns = ('pk', 'id', 'scope', 'address_family')


class BGPPeerTable(TenancyColumnsMixin, NetBoxTable):
    name = tables.Column(linkify=True, verbose_name=_('Name'))
    scope = tables.Column(linkify=True, verbose_name=_('Scope'))
    peer = tables.Column(linkify=True, verbose_name=_('Scope'))
    source = tables.Column(linkify=True, verbose_name=_('Scope'))
    peer_group = tables.Column(linkify=True, verbose_name=_('Scope'))
    peer_session = tables.Column(linkify=True, verbose_name=_('Scope'))
    remote_as = tables.Column(linkify=True, verbose_name=_('Scope'))
    local_as = tables.Column(linkify=True, verbose_name=_('Scope'))

    class Meta(NetBoxTable.Meta):
        model = BGPPeer
        fields = (
            'pk',
            'id',
            'name',
            'scope',
            'peer',
            'source',
            'peer_group',
            'peer_session',
            'remote_as',
            'status',
            'local_as',
            'bfd',
            'password',
            'tenant_group',
            'tenant',
            'ttl',
            'address_families',
        )
        default_columns = (
            'pk',
            'id',
            'name',
            'scope',
            'peer',
            'remote_as',
            'enabled',
            'status',
        )


class BGPPeerAddressFamilyTable(TenancyColumnsMixin, NetBoxTable):
    assigned_object_type = columns.ContentTypeColumn(verbose_name=_('Object Type'))
    assigned_object = tables.Column(
        linkify=True, orderable=False, verbose_name=_('Object')
    )
    address_family = tables.Column(
        linkify=True,
        verbose_name=_('Address Family'),
    )
    peer_policy = tables.Column(
        linkify=True,
        verbose_name=_('Peer Policy'),
    )
    prefixlist_in = tables.Column(
        linkify=True,
        verbose_name=_('Prefix List (in)'),
    )
    prefixlist_out = tables.Column(
        linkify=True,
        verbose_name=_('Prefix List (out)'),
    )
    routemap_in = tables.Column(
        linkify=True,
        verbose_name=_('Route Map (in)'),
    )
    routemap_out = tables.Column(
        linkify=True,
        verbose_name=_('Route Map (out)'),
    )

    class Meta(NetBoxTable.Meta):
        model = BGPPeerAddressFamily
        fields = (
            'pk',
            'id',
            'assigned_object_type',
            'assigned_object',
            'address_family',
            'peer_policy',
            'enabled',
            'prefixlist_in',
            'prefixlist_out',
            'routemap_in',
            'routemap_out',
        )
        default_columns = (
            'pk',
            'id',
            'assigned_object_type',
            'assigned_object',
            'address_family',
            'enabled',
        )


class BFDProfileTable(TenancyColumnsMixin, NetBoxTable):
    name = tables.Column(linkify=True, verbose_name=_('Name'))

    class Meta(NetBoxTable.Meta):
        model = BFDProfile
        fields = (
            'pk',
            'id',
            'name',
            'min_rx_int',
            'min_tx_int',
            'multiplier',
            'hold',
            'tenant',
        )
        default_columns = (
            'pk',
            'id',
            'name',
        )



from netbox.api.viewsets import NetBoxModelViewSet
from netbox_routing import filtersets
from netbox_routing.api._serializers.objects import *
from netbox_routing.models.objects import *

__all__ = (
    'PrefixListViewSet',
    'PrefixListEntryViewSet',
    'RouteMapViewSet',
    'RouteMapEntryViewSet',
    'ASPathViewSet',
    'ASPathEntryViewSet',
)


class PrefixListViewSet(NetBoxModelViewSet):
    queryset = PrefixList.objects.all()
    serializer_class = PrefixListSerializer
    filterset_class = filtersets.PrefixListFilterSet


class PrefixListEntryViewSet(NetBoxModelViewSet):
    queryset = PrefixListEntry.objects.all()
    serializer_class = PrefixListEntrySerializer
    filterset_class = filtersets.PrefixListEntryFilterSet


class CustomPrefixViewSet(NetBoxModelViewSet):
    queryset = CustomPrefix.objects.all()
    serializer_class = CustomPrefixSerializer
    filterset_class = filtersets.CustomPrefixFilterSet


class RouteMapViewSet(NetBoxModelViewSet):
    queryset = RouteMap.objects.all()
    serializer_class = RouteMapSerializer
    filterset_class = filtersets.RouteMapFilterSet


class RouteMapEntryViewSet(NetBoxModelViewSet):
    queryset = RouteMapEntry.objects.all()
    serializer_class = RouteMapEntrySerializer
    filterset_class = filtersets.RouteMapEntryFilterSet


class ASPathViewSet(NetBoxModelViewSet):
    queryset = ASPath.objects.all()
    serializer_class = ASPathSerializer
    filterset_class = filtersets.ASPathFilterSet


class ASPathEntryViewSet(NetBoxModelViewSet):
    queryset = ASPathEntry.objects.all()
    serializer_class = ASPathEntrySerializer
    filterset_class = filtersets.ASPathEntryFilterSet



from netbox.search import SearchIndex, register_search
from netbox_routing.models.static import StaticRoute

__all__ = ('StaticRouteIndex',)


@register_search
class StaticRouteIndex(SearchIndex):
    model = StaticRoute
    fields = (
        ('name', 100),
        ('prefix', 200),
        ('next_hop', 300),
        ('comments', 5000),
    )
    display_attrs = ('prefix', 'next_hop', 'vrf')



from dcim.models import Interface
from ipam.models import VRF
from utilities.testing import APIViewTestCases, create_test_device

from netbox_routing.models import OSPFInstance, OSPFArea, OSPFInterface
from netbox_routing.tests.base import IPAddressFieldMixin

__all__ = (
    'OSPFInstanceTestCase',
    'OSPFAreaTestCase',
    'OSPFInterfaceTestCase',
)


class OSPFInstanceTestCase(IPAddressFieldMixin, APIViewTestCases.APIViewTestCase):
    model = OSPFInstance
    view_namespace = 'plugins-api:netbox_routing'
    brief_fields = [
        'device',
        'display',
        'id',
        'name',
        'process_id',
        'router_id',
        'url',
        'vrf',
    ]

    user_permissions = ('dcim.view_device',)

    bulk_update_data = {'description': "A test description"}

    @classmethod
    def setUpTestData(cls):
        vrf = VRF.objects.create(name='Test VRF')
        device = create_test_device(name='Test Device')

        data = (
            cls.model(
                name='Instance 1',
                device=device,
                router_id='1.1.1.1',
                process_id=1,
                vrf=None,
            ),
            cls.model(
                name='Instance 2',
                device=device,
                router_id='2.2.2.2',
                process_id=2,
                vrf=vrf,
            ),
            cls.model(
                name='Instance 3',
                device=device,
                router_id='3.3.3.3',
                process_id=3,
                vrf=None,
            ),
        )
        cls.model.objects.bulk_create(data)

        cls.create_data = [
            {
                'name': 'Instance X',
                'device': device.pk,
                'router_id': '4.4.4.4',
                'process_id': 4,
                'vrf': vrf.pk,
            },
        ]


class OSPFAreaTestCase(IPAddressFieldMixin, APIViewTestCases.APIViewTestCase):
    model = OSPFArea
    view_namespace = 'plugins-api:netbox_routing'
    brief_fields = [
        'area_id',
        'area_type',
        'display',
        'id',
        'url',
    ]

    bulk_update_data = {'description': "A test description"}

    @classmethod
    def setUpTestData(cls):

        data = (
            cls.model(area_id='1', area_type='stub'),
            cls.model(area_id='2', area_type='stub'),
            cls.model(area_id='3', area_type='stub'),
        )
        cls.model.objects.bulk_create(data)

        cls.create_data = [
            {
                'area_id': '4',
                'area_type': 'stub',
            },
        ]


class OSPFInterfaceTestCase(IPAddressFieldMixin, APIViewTestCases.APIViewTestCase):
    model = OSPFInterface
    view_namespace = 'plugins-api:netbox_routing'
    brief_fields = [
        'area',
        'display',
        'id',
        'instance',
        'interface',
        'passive',
        'url',
    ]

    user_permissions = (
        'netbox_routing.view_ospfinstance',
        'netbox_routing.view_ospfarea',
        'dcim.view_device',
        'dcim.view_interface',
    )

    bulk_update_data = {'description': "A test description"}

    @classmethod
    def setUpTestData(cls):

        device = create_test_device(name='Test Device')
        instance = OSPFInstance.objects.create(
            name='Instance 1', device=device, router_id='1.1.1.1', process_id=1
        )
        area = OSPFArea.objects.create(area_id='0')

        interfaces = (
            Interface(device=device, name='Interface 1', type='virtual'),
            Interface(device=device, name='Interface 2', type='virtual'),
            Interface(device=device, name='Interface 3', type='virtual'),
            Interface(device=device, name='Interface 4', type='virtual'),
        )
        Interface.objects.bulk_create(interfaces)

        data = (
            cls.model(
                instance=instance, area=area, interface=interfaces[0], passive=True
            ),
            cls.model(
                instance=instance, area=area, interface=interfaces[1], passive=False
            ),
            cls.model(
                instance=instance,
                area=area,
                interface=interfaces[2],
            ),
        )
        cls.model.objects.bulk_create(data)

        cls.create_data = [
            {
                'instance': instance.pk,
                'area': area.pk,
                'interface': interfaces[3].pk,
                'passive': True,
                'priority': 2,
                'authentication': 'message-digest',
                'passphrase': 'test-password',
                'bfd': True,
            },
        ]



import django_tables2 as tables
from django.utils.translation import gettext_lazy as _

from netbox.tables import NetBoxTable
from netbox_routing.models import OSPFArea, OSPFInstance, OSPFInterface

__all__ = (
    'OSPFAreaTable',
    'OSPFInstanceTable',
    'OSPFInterfaceTable',
)


class OSPFInstanceTable(NetBoxTable):
    device = tables.Column(verbose_name=_('Device'), linkify=True)
    vrf = tables.Column(verbose_name=_('VRF'), linkify=True)

    class Meta(NetBoxTable.Meta):
        model = OSPFInstance
        fields = (
            'pk',
            'id',
            'name',
            'router_id',
            'process_id',
            'device',
            'vrf',
        )
        default_columns = (
            'pk',
            'id',
            'name',
            'router_id',
            'process_id',
            'device',
        )


class OSPFAreaTable(NetBoxTable):
    area_id = tables.Column(verbose_name=_('Area ID'), linkify=True)

    class Meta(NetBoxTable.Meta):
        model = OSPFArea
        fields = ('pk', 'id', 'area_id', 'area_type')
        default_columns = ('pk', 'id', 'area_id', 'area_type')


class OSPFInterfaceTable(NetBoxTable):
    instance = tables.Column(verbose_name=_('Instance'), linkify=True)
    device = tables.Column(
        verbose_name=_('Device'),
        linkify=True,
        accessor='instance__device',
    )
    area = tables.Column(verbose_name=_('Area'), linkify=True)
    interface = tables.Column(verbose_name=_('Interface'), linkify=True)

    class Meta(NetBoxTable.Meta):
        model = OSPFInterface
        fields = (
            'pk',
            'id',
            'instance',
            'device',
            'area',
            'interface',
            'passive',
            'priority',
            'bfd',
            'authentication',
            'passphrase',
        )
        default_columns = ('pk', 'id', 'instance', 'area', 'interface', 'passive')



from django import forms
from django.utils.translation import gettext as _

from dcim.models import Device
from ipam.models import VRF
from netbox.forms import NetBoxModelBulkEditForm
from utilities.forms import BOOLEAN_WITH_BLANK_CHOICES, add_blank_choice
from utilities.forms.fields import DynamicModelChoiceField, CommentField

from netbox_routing import choices
from netbox_routing.choices.ospf import OSPFAreaTypeChoices
from netbox_routing.models import OSPFArea, OSPFInstance, OSPFInterface

__all__ = (
    'OSPFInterfaceBulkEditForm',
    'OSPFInstanceBulkEditForm',
    'OSPFAreaBulkEditForm',
)

from utilities.forms.rendering import FieldSet


class OSPFInstanceBulkEditForm(NetBoxModelBulkEditForm):
    device = DynamicModelChoiceField(
        queryset=Device.objects.all(), label=_('Device'), required=False, selector=True
    )
    vrf = DynamicModelChoiceField(
        queryset=VRF.objects.all(), label=_('VRF'), required=False, selector=True
    )

    description = forms.CharField(
        label=_('Description'), max_length=200, required=False
    )
    comments = CommentField()

    model = OSPFInstance
    fieldsets = (
        FieldSet('device', 'vrf', name='OSPF'),
        FieldSet(
            'description',
        ),
    )
    nullable_fields = (
        'vrf',
        'description',
    )


class OSPFAreaBulkEditForm(NetBoxModelBulkEditForm):
    area_type = forms.ChoiceField(
        label=_('Area Type'), choices=OSPFAreaTypeChoices, required=False
    )
    description = forms.CharField(
        label=_('Description'), max_length=200, required=False
    )
    comments = CommentField()

    model = OSPFArea
    fieldsets = (FieldSet('description'),)
    nullable_fields = ()


class OSPFInterfaceBulkEditForm(NetBoxModelBulkEditForm):
    instance = DynamicModelChoiceField(
        queryset=OSPFInstance.objects.all(),
        label=_('OSPF Instance'),
        required=False,
        selector=True,
    )
    area = DynamicModelChoiceField(
        queryset=OSPFArea.objects.all(),
        label=_('OSPF Area'),
        required=False,
        selector=True,
    )
    passive = forms.ChoiceField(
        label=_('Passive'), choices=BOOLEAN_WITH_BLANK_CHOICES, required=False
    )
    priority = forms.IntegerField(label=_('Priority'), required=False)
    bfd = forms.ChoiceField(
        label=_('BFD'), choices=BOOLEAN_WITH_BLANK_CHOICES, required=False
    )
    authentication = forms.ChoiceField(
        label=_('Authentication'),
        choices=add_blank_choice(choices.AuthenticationChoices),
        required=False,
    )
    passphrase = forms.CharField(label=_('Passphrase'), required=False)

    description = forms.CharField(
        label=_('Description'), max_length=200, required=False
    )
    comments = CommentField()

    model = OSPFInterface
    fieldsets = (
        FieldSet('instance', 'area', name='OSPF'),
        FieldSet(
            'passive',
            'priority',
            'bfd',
            'authentication',
            'passphrase',
            name='Attributes',
        ),
        FieldSet('description'),
    )
    nullable_fields = ()



# Generated by Django 4.2.4 on 2024-05-07 15:07

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0006_bgp'),
    ]

    operations = [
        migrations.AddField(
            model_name='bgpsessiontemplate',
            name='asn',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name='session_templates',
                to='ipam.asn',
            ),
        ),
        migrations.AddField(
            model_name='bgpsessiontemplate',
            name='bfd',
            field=models.CharField(blank=True, max_length=50, null=True),
        ),
        migrations.AddField(
            model_name='bgpsessiontemplate',
            name='parent',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name='children',
                to='netbox_routing.bgpsessiontemplate',
            ),
        ),
        migrations.AddField(
            model_name='bgpsessiontemplate',
            name='password',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]



from django import forms
from django.utils.translation import gettext as _

from dcim.models import Device
from ipam.models import ASN, VRF, IPAddress
from netbox.forms import NetBoxModelFilterSetForm
from tenancy.forms import TenancyFilterForm
from utilities.forms.fields import TagFilterField, DynamicModelMultipleChoiceField
from utilities.forms.rendering import FieldSet

from netbox_routing.choices.bgp import *
from netbox_routing.models.bgp import *

__all__ = (
    'BGPSettingFilterForm',
    'BGPRouterFilterForm',
    'BGPScopeFilterForm',
    'BGPAddressFamilyFilterForm',
    'BGPPeerFilterForm',
    'BGPPeerAddressFamilyFilterForm',
    'BGPPeerTemplateFilterForm',
    'BGPPolicyTemplateFilterForm',
    'BGPSessionTemplateFilterForm',
    'BFDProfileFilterForm',
)


class BGPSettingFilterForm(NetBoxModelFilterSetForm):
    model = BGPSetting
    fieldsets = (FieldSet('q', 'filter_id', 'tag'),)
    tag = TagFilterField(model)


class BGPPeerTemplateFilterForm(NetBoxModelFilterSetForm):
    model = BGPPeerTemplate
    fieldsets = (FieldSet('q', 'filter_id', 'tag'),)
    tag = TagFilterField(model)


class BGPPolicyTemplateFilterForm(NetBoxModelFilterSetForm):
    model = BGPPolicyTemplate
    fieldsets = (FieldSet('q', 'filter_id', 'tag'),)
    tag = TagFilterField(model)


class BGPSessionTemplateFilterForm(NetBoxModelFilterSetForm):
    model = BGPSessionTemplate
    fieldsets = (FieldSet('q', 'filter_id', 'tag'),)
    tag = TagFilterField(model)


class BGPRouterFilterForm(
    TenancyFilterForm,
    # ContactModelFilterForm,
    NetBoxModelFilterSetForm,
):
    device_id = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False,
        selector=True,
        label=_('Device'),
    )
    asn_id = DynamicModelMultipleChoiceField(
        queryset=ASN.objects.all(),
        required=False,
        selector=True,
        label=_('ASN'),
    )
    model = BGPRouter
    fieldsets = (FieldSet('q', 'filter_id', 'tag', 'device_id', 'asn_id'),)
    tag = TagFilterField(model)


class BGPScopeFilterForm(
    TenancyFilterForm,
    # ContactModelFilterForm,
    NetBoxModelFilterSetForm,
):
    router_id = DynamicModelMultipleChoiceField(
        queryset=BGPRouter.objects.all(),
        required=False,
        selector=True,
        label=_('Router'),
    )
    vrf_id = DynamicModelMultipleChoiceField(
        queryset=VRF.objects.all(),
        required=False,
        selector=True,
        label=_('VRF'),
    )
    model = BGPScope
    fieldsets = (FieldSet('q', 'filter_id', 'tag', 'router_id', 'vrf_id'),)
    tag = TagFilterField(model)


class BGPAddressFamilyFilterForm(TenancyFilterForm, NetBoxModelFilterSetForm):
    scope_id = DynamicModelMultipleChoiceField(
        queryset=BGPScope.objects.all(),
        required=False,
        selector=True,
        label=_('Router'),
    )
    address_family = forms.MultipleChoiceField(
        choices=BGPAddressFamilyChoices,
        required=False,
        label=_('Address Family'),
    )
    model = BGPAddressFamily
    fieldsets = (FieldSet('q', 'filter_id', 'tag', 'scope_id', 'address_family'),)
    tag = TagFilterField(model)


class BGPPeerFilterForm(
    TenancyFilterForm,
    # ContactModelFilterForm,
    NetBoxModelFilterSetForm,
):
    scope_id = DynamicModelMultipleChoiceField(
        queryset=BGPScope.objects.all(),
        required=False,
        selector=True,
        label=_('Scope'),
    )
    peer_id = DynamicModelMultipleChoiceField(
        queryset=IPAddress.objects.all(),
        required=False,
        selector=True,
        label=_('Peer'),
    )
    peer_group_id = DynamicModelMultipleChoiceField(
        queryset=BGPPeerTemplate.objects.all(),
        required=False,
        selector=True,
        label=_('Peer Group'),
    )
    remote_as_id = DynamicModelMultipleChoiceField(
        queryset=ASN.objects.all(),
        required=False,
        selector=True,
        label=_('Remote AS'),
    )
    local_as_id = DynamicModelMultipleChoiceField(
        queryset=ASN.objects.all(),
        required=False,
        selector=True,
        label=_('Local AS'),
    )
    model = BGPPeer
    fieldsets = (
        FieldSet(
            'q',
            'filter_id',
            'tag',
            'scope_id',
            'peer_id',
            'peer_group_id',
            'remote_as_id',
            'local_as_id',
            'status',
            'enabled',
        ),
    )
    tag = TagFilterField(model)


class BGPPeerAddressFamilyFilterForm(
    TenancyFilterForm,
    # ContactModelFilterForm,
    NetBoxModelFilterSetForm,
):
    peer_id = DynamicModelMultipleChoiceField(
        queryset=BGPPeer.objects.all(),
        required=False,
        selector=True,
        label=_('Peer'),
    )
    peer_group_id = DynamicModelMultipleChoiceField(
        queryset=BGPPeerTemplate.objects.all(),
        required=False,
        selector=True,
        label=_('Peer Group'),
    )
    address_family_id = DynamicModelMultipleChoiceField(
        queryset=BGPAddressFamily.objects.all(),
        required=False,
        selector=True,
        label=_('Address Family'),
    )
    model = BGPPeerAddressFamily
    fieldsets = (
        FieldSet(
            'q',
            'filter_id',
            'tag',
            'peer_id',
            'peer_group_id',
            'address_family_id',
            'enabled',
        ),
    )
    tag = TagFilterField(model)


class BFDProfileFilterForm(
    TenancyFilterForm,
    # ContactModelFilterForm,
    NetBoxModelFilterSetForm,
):
    model = BFDProfile
    fieldsets = (
        FieldSet(
            'q',
            'filter_id',
            'tag',
        ),
    )
    tag = TagFilterField(model)



from django import forms
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.forms import fields
from django.utils.translation import gettext as _

from dcim.models import Device, Region, SiteGroup, Site, Location
from ipam.models import ASN, VRF, IPAddress
from netbox.forms import PrimaryModelForm
from tenancy.forms import TenancyForm
from utilities.forms.fields import (
    DynamicModelChoiceField,
    DynamicModelMultipleChoiceField,
)
from utilities.forms.rendering import FieldSet, TabbedGroups
from virtualization.models import Cluster, VirtualMachine, ClusterGroup

from netbox_routing.choices.bgp import *
from netbox_routing.models.objects import *
from netbox_routing.models.bgp import *

__all__ = (
    'BGPSettingForm',
    'BGPRouterForm',
    'BGPScopeForm',
    'BGPAddressFamilyForm',
    'BGPPeerForm',
    'BGPPeerAddressFamilyForm',
    'BGPPeerTemplateForm',
    'BGPPolicyTemplateForm',
    'BGPSessionTemplateForm',
    'BFDProfileForm',
)


class BGPSettingMixin:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._append_settings_fields()

    def _append_settings_fields(self):
        fieldsets = list(self.fieldsets)
        assigned_fields = []
        setting_fields = [
            'router_id',
            'asdot',
            'timers_keepalive',
            'timers_hold',
            'auto_summary',
            'default_metric',
            'default_information_originate',
            'distance_ebgp',
            'distance_ibgp',
            'distance_embgp',
            'distance_imbgp',
            'paths_maximum',
            'paths_maximum_secondary',
            'graceful_restart',
            'additional_paths_receive',
            'additional_paths_send',
            'additional_paths_install',
            'test',
        ]
        for key, label in BGPSettingChoices.CHOICES:
            initial = None
            if hasattr(self, 'instance'):
                setting = BGPSetting.objects.filter(
                    assigned_object_type=ContentType.objects.get_for_model(
                        self.Meta.model
                    ),
                    assigned_object_id=self.instance.pk,
                    key=key,
                ).first()
                if setting:
                    initial = setting.value
            if BGPSettingChoices.FIELD_TYPES[key] == 'ipaddr':
                self.fields[key] = fields.CharField(
                    label=label, required=False, initial=initial, max_length=128
                )
                css = self.fields[key].widget.attrs.get('class', '')
                self.fields[key].widget.attrs['class'] = f'{css} form-control'
            elif BGPSettingChoices.FIELD_TYPES[key] == 'integer':
                self.fields[key] = fields.IntegerField(
                    label=label,
                    required=False,
                    initial=initial,
                    min_value=0,
                    max_value=65535,
                )
                css = self.fields[key].widget.attrs.get('class', '')
                self.fields[key].widget.attrs['class'] = f'{css} form-control'
            elif BGPSettingChoices.FIELD_TYPES[key] == 'boolean':
                choices = (
                    (None, '---------'),
                    (True, _('True')),
                    (False, _('False')),
                )
                self.fields[key] = fields.NullBooleanField(
                    label=label,
                    required=False,
                    initial=initial,
                    widget=forms.Select(choices=choices),
                )
                css = self.fields[key].widget.attrs.get('class', '')
                self.fields[key].widget.attrs['class'] = f'{css} form-control'
            assigned_fields.append(key)
            if key not in setting_fields:
                setting_fields.append(key)

        fieldset = FieldSet(
            *setting_fields,
            name=_('Settings'),
        )
        fieldsets.append(fieldset)
        if fieldset not in self.fieldsets:
            self.fieldsets = fieldsets

    def _clean_fieldset(self):
        pass

    def save(self, *args, **kwargs):
        settings = {}
        for key, name in BGPSettingChoices.CHOICES:
            if key in self.cleaned_data:
                settings[key] = self.cleaned_data.pop(key)
        obj = super().save(*args, **kwargs)

        for key, name in BGPSettingChoices.CHOICES:
            value = settings.get(key, None)
            setting = BGPSetting.objects.filter(
                assigned_object_type=self.get_assigned_object_type(),
                assigned_object_id=self.get_assigned_object_id(),
                key=key,
            ).first()
            # print(f'{key}')
            if setting and value:
                # print('\tPrev: {setting.value}')
                # print('\tNew: {value}')
                setting.value = settings.get(key)
                setting.clean()
                setting.save()
            elif value:
                # print('\tNew: {value}')
                setting = BGPSetting(
                    assigned_object=self.instance,
                    key=key,
                    value=settings.get(key, None),
                )
                setting.clean()
                setting.save()
            elif setting:
                # print('\tPrev: {setting.value}')
                setting.delete()

        return obj

    def get_assigned_object_type(self):
        return ContentType.objects.get_for_model(self.instance).pk

    def get_assigned_object_id(self):
        return self.instance.pk


class BGPSettingForm(PrimaryModelForm):

    router = DynamicModelChoiceField(
        queryset=BGPRouter.objects.all(),
        required=True,
        selector=True,
        label=_('Router'),
    )
    scope = DynamicModelChoiceField(
        queryset=BGPScope.objects.all(),
        required=True,
        selector=True,
        label=_('Scope'),
    )
    addressfamily = DynamicModelChoiceField(
        queryset=BGPAddressFamily.objects.all(),
        required=True,
        selector=True,
        label=_('Address Family'),
    )
    key = forms.ChoiceField(
        choices=BGPSettingChoices,
        required=True,
        label=_('Setting Name'),
    )
    value = forms.CharField(
        required=True,
        label=_('Setting Value'),
    )

    fieldsets = (
        FieldSet(
            TabbedGroups(
                FieldSet('router', name=_('Router')),
                FieldSet('scope', name=_('Scope')),
                FieldSet('addressfamily', name=_('Address Family')),
            ),
            name='Assigned Object',
        ),
        FieldSet('key', 'value', name=_('Settings')),
    )

    class Meta:
        model = BGPSetting
        fields = [
            'router',
            'scope',
            'addressfamily',
            'key',
            'value',
            'tags',
            'owner',
        ]

    def __init__(self, *args, **kwargs):

        # Initialize helper selectors
        instance = kwargs.get('instance')
        initial = kwargs.get('initial', {}).copy()
        if instance:
            if type(instance.assigned_object) is BGPRouter:
                initial['router'] = instance.assigned_object
            elif type(instance.assigned_object) is BGPScope:
                initial['scope'] = instance.assigned_object
            elif type(instance.assigned_object) is BGPAddressFamily:
                initial['addressfamily'] = instance.assigned_object
        kwargs['initial'] = initial
        super().__init__(*args, **kwargs)

    def clean(self):
        super().clean()

        # Handle object assignment
        selected_objects = [
            field
            for field in ('router', 'scope', 'addressfamily')
            if self.cleaned_data[field]
        ]
        if len(selected_objects) > 1:
            raise forms.ValidationError(
                {
                    selected_objects[1]: _(
                        "A BGP Setting can only be assigned to a single object."
                    )
                }
            )
        elif selected_objects:
            self.instance.assigned_object = self.cleaned_data[selected_objects[0]]
        else:
            raise ValidationError(_('A BGP Setting must be assigned to an object.'))


class BGPPeerTemplateForm(TenancyForm, PrimaryModelForm):
    remote_as = DynamicModelChoiceField(
        queryset=ASN.objects.all(),
        required=False,
        selector=True,
        label=_('Remote ASN'),
    )

    fieldsets = (
        FieldSet('name', 'remote_as', 'enabled'),
        FieldSet('tenant_group', 'tenant', name=_('Tenancy')),
    )

    class Meta:
        model = BGPPeerTemplate
        fields = [
            'name',
            'remote_as',
            'tenant_group',
            'tenant',
            'tags',
            'owner',
        ]


class BGPPolicyTemplateForm(TenancyForm, PrimaryModelForm):
    parents = DynamicModelMultipleChoiceField(
        queryset=BGPPeerTemplate.objects.all(),
        required=False,
        selector=True,
        label=_('Parent Policy Templates'),
    )
    prefixlist_out = DynamicModelChoiceField(
        queryset=PrefixList.objects.all(),
        required=False,
        selector=True,
        label=_('Prefix List (out)'),
    )
    prefixlist_in = DynamicModelChoiceField(
        queryset=PrefixList.objects.all(),
        required=False,
        selector=True,
        label=_('Prefix List (in)'),
    )
    routemap_out = DynamicModelChoiceField(
        queryset=RouteMap.objects.all(),
        required=False,
        selector=True,
        label=_('Route Map (out)'),
    )
    routemap_in = DynamicModelChoiceField(
        queryset=RouteMap.objects.all(),
        required=False,
        selector=True,
        label=_('Route Map (in)'),
    )

    fieldsets = (
        FieldSet(
            'name',
            'parents',
        ),
        FieldSet(
            'prefixlist_out',
            'prefixlist_in',
            'routemap_out',
            'routemap_in',
            name=_('Setings'),
        ),
        FieldSet('tenant_group', 'tenant', name=_('Tenancy')),
    )

    class Meta:
        model = BGPPolicyTemplate
        fields = [
            'name',
            'parents',
            'prefixlist_out',
            'prefixlist_in',
            'routemap_out',
            'routemap_in',
            'tenant_group',
            'tenant',
            'tags',
            'owner',
        ]


class BGPSessionTemplateForm(TenancyForm, PrimaryModelForm):
    parent = DynamicModelChoiceField(
        queryset=BGPSessionTemplate.objects.all(),
        required=False,
        selector=True,
        label=_('Parent Peer Template'),
    )
    remote_as = DynamicModelChoiceField(
        queryset=ASN.objects.all(),
        required=False,
        selector=True,
        label=_('Remote ASN'),
    )
    local_as = DynamicModelChoiceField(
        queryset=ASN.objects.all(),
        required=False,
        selector=True,
        label=_('Local ASN'),
    )

    fieldsets = (
        FieldSet(
            'name',
            'parent',
            'remote_as',
            'local_as',
        ),
        FieldSet('pasword', 'bfd', 'ttl', 'enabled', name=_('Setings')),
        FieldSet('tenant_group', 'tenant', name=_('Tenancy')),
    )

    class Meta:
        model = BGPSessionTemplate
        fields = [
            'name',
            'parent',
            'enabled',
            'remote_as',
            'local_as',
            'bfd',
            'ttl',
            'password',
            'tenant_group',
            'tenant',
            'tags',
            'owner',
        ]


class BGPRouterForm(BGPSettingMixin, TenancyForm, PrimaryModelForm):
    region = DynamicModelChoiceField(
        queryset=Region.objects.all(),
        required=False,
        selector=True,
        label=_('Region'),
    )
    site_group = DynamicModelChoiceField(
        queryset=SiteGroup.objects.all(),
        required=False,
        selector=True,
        label=_('Site Group'),
    )
    site = DynamicModelChoiceField(
        queryset=Site.objects.all(),
        required=False,
        selector=True,
        label=_('Site'),
    )
    location = DynamicModelChoiceField(
        queryset=Location.objects.all(),
        required=False,
        selector=True,
        label=_('Location'),
    )
    device = DynamicModelChoiceField(
        queryset=Device.objects.all(),
        required=False,
        selector=True,
        label=_('Device'),
    )
    cluster_group = DynamicModelChoiceField(
        queryset=ClusterGroup.objects.all(),
        required=False,
        selector=True,
        label=_('Cluster Group'),
    )
    cluster = DynamicModelChoiceField(
        queryset=Cluster.objects.all(),
        required=False,
        selector=True,
        label=_('Cluster'),
    )
    virtual_machine = DynamicModelChoiceField(
        queryset=VirtualMachine.objects.all(),
        required=False,
        selector=True,
        label=_('Device'),
    )
    asn = DynamicModelChoiceField(
        queryset=ASN.objects.all(),
        required=True,
        selector=True,
        label=_('ASN'),
    )
    policy_templates = DynamicModelMultipleChoiceField(
        queryset=BGPPolicyTemplate.objects.all(),
        required=False,
        selector=False,
        label=_('Policy Templates'),
    )
    session_templates = DynamicModelMultipleChoiceField(
        queryset=BGPSessionTemplate.objects.all(),
        required=False,
        selector=False,
        label=_('Session Templates'),
    )
    peer_templates = DynamicModelMultipleChoiceField(
        queryset=BGPPeerTemplate.objects.all(),
        required=False,
        selector=False,
        label=_('Peer Templates'),
    )

    fieldsets = [
        FieldSet(
            'name',
        ),
        FieldSet(
            TabbedGroups(
                FieldSet(
                    'region', 'site_group', 'site', 'location', name=_('Locations')
                ),
                FieldSet('device', name=_('Device')),
                FieldSet(
                    'cluster_group',
                    'cluster',
                    'virtual_machine',
                    name=_('Virtualization'),
                ),
            ),
            name=_('Assigned Object'),
        ),
        FieldSet('asn', name=_('Router')),
        FieldSet(
            'peer_templates',
            'policy_templates',
            'session_templates',
            name=_('Templates'),
        ),
        FieldSet('tenant_group', 'tenant', name=_('Tenancy')),
    ]

    class Meta:
        model = BGPRouter
        fields = [
            'name',
            'region',
            'site_group',
            'site',
            'location',
            'device',
            'cluster_group',
            'cluster',
            'virtual_machine',
            'asn',
            'policy_templates',
            'session_templates',
            'peer_templates',
            'tenant_group',
            'tenant',
            'tags',
            'owner',
        ]

    def __init__(self, *args, **kwargs):

        # Initialize helper selectors
        instance = kwargs.get('instance')
        initial = kwargs.get('initial', {}).copy()
        if instance:
            if type(instance.assigned_object) is Region:
                initial['region'] = instance.assigned_object
            elif type(instance.assigned_object) is SiteGroup:
                initial['site_group'] = instance.assigned_object
            elif type(instance.assigned_object) is Site:
                initial['site'] = instance.assigned_object
            elif type(instance.assigned_object) is Location:
                initial['location'] = instance.assigned_object
            elif type(instance.assigned_object) is Device:
                initial['device'] = instance.assigned_object
            elif type(instance.assigned_object) is ClusterGroup:
                initial['cluster_group'] = instance.assigned_object
            elif type(instance.assigned_object) is Cluster:
                initial['cluster'] = instance.assigned_object
            elif type(instance.assigned_object) is VirtualMachine:
                initial['virtual_machine'] = instance.assigned_object
        kwargs['initial'] = initial
        super().__init__(*args, **kwargs)

    def clean(self):
        super().clean()

        # Handle object assignment
        selected_objects = [
            field
            for field in (
                'region',
                'site_group',
                'site',
                'location',
                'device',
                'cluster_group',
                'cluster',
                'virtual_machine',
            )
            if self.cleaned_data[field]
        ]
        if len(selected_objects) > 1:
            raise forms.ValidationError(
                {
                    selected_objects[1]: _(
                        "A BGP Router can only be assigned to a single object."
                    )
                }
            )
        elif selected_objects:
            self.instance.assigned_object = self.cleaned_data[selected_objects[0]]
        else:
            self.instance.assigned_object = None

    def save(self, *args, **kwargs):
        return super().save(*args, **kwargs)


class BGPScopeForm(BGPSettingMixin, TenancyForm, PrimaryModelForm):
    router = DynamicModelChoiceField(
        queryset=BGPRouter.objects.all(),
        required=True,
        selector=True,
        label=_('Router'),
    )
    vrf = DynamicModelChoiceField(
        queryset=VRF.objects.all(),
        required=False,
        selector=True,
        label=_('VRF'),
    )

    fieldsets = (
        FieldSet('router', 'vrf', name=_('Scope')),
        FieldSet('tenant_group', 'tenant', name=_('Tenancy')),
    )

    class Meta:
        model = BGPScope
        fields = [
            'router',
            'vrf',
            'tenant_group',
            'tenant',
            'tags',
            'owner',
        ]

    def save(self, *args, **kwargs):
        return super().save(*args, **kwargs)


class BGPAddressFamilyForm(BGPSettingMixin, TenancyForm, PrimaryModelForm):
    scope = DynamicModelChoiceField(
        queryset=BGPScope.objects.all(),
        required=True,
        selector=True,
        label=_('Scope'),
    )
    address_family = forms.ChoiceField(
        choices=BGPAddressFamilyChoices,
        required=True,
        label=_('Address Family'),
    )

    fieldsets = (
        FieldSet('scope', 'address_family', name=_('Address Family')),
        FieldSet('tenant_group', 'tenant', name=_('Tenancy')),
    )

    class Meta:
        model = BGPAddressFamily
        fields = [
            'scope',
            'address_family',
            'tenant_group',
            'tenant',
            'tags',
            'owner',
        ]

    def save(self, *args, **kwargs):
        return super().save(*args, **kwargs)


class BGPPeerForm(BGPSettingMixin, TenancyForm, PrimaryModelForm):
    scope = DynamicModelChoiceField(
        queryset=BGPScope.objects.all(),
        required=True,
        selector=True,
        label=_('Scope'),
    )
    peer = DynamicModelChoiceField(
        queryset=IPAddress.objects.all(),
        required=True,
        selector=True,
        label=_('Peer'),
    )
    remote_as = DynamicModelChoiceField(
        queryset=ASN.objects.all(),
        required=False,
        selector=True,
        label=_('Remote AS'),
    )
    local_as = DynamicModelChoiceField(
        queryset=ASN.objects.all(),
        required=False,
        selector=True,
        label=_('Local AS'),
    )

    fieldsets = (
        FieldSet(
            'name',
        ),
        FieldSet('scope', 'peer', 'status', name=_('Peer')),
        FieldSet('remote_as', 'local_as', name=_('ASNs')),
        FieldSet('enabled', 'bfd', 'password', 'ttl', name=_('Peer Settings')),
        FieldSet('tenant_group', 'tenant', name=_('Tenancy')),
    )

    class Meta:
        model = BGPPeer
        fields = [
            'name',
            'scope',
            'peer',
            'remote_as',
            'local_as',
            'tenant',
            'enabled',
            'status',
            'ttl',
            'bfd',
            'password',
            'tenant_group',
            'tenant',
            'tags',
            'owner',
        ]

    def save(self, *args, **kwargs):
        return super().save(*args, **kwargs)


class BGPPeerAddressFamilyForm(BGPSettingMixin, TenancyForm, PrimaryModelForm):
    peer = DynamicModelChoiceField(
        queryset=BGPPeer.objects.all(),
        required=False,
        selector=True,
        label=_('Peer'),
    )
    peer_group = DynamicModelChoiceField(
        queryset=BGPPeerTemplate.objects.all(),
        required=False,
        selector=True,
        label=_('Peer Group'),
    )
    address_family = DynamicModelChoiceField(
        queryset=BGPAddressFamily.objects.all(),
        required=True,
        selector=True,
        label=_('Address Family'),
    )

    prefix_list_in = DynamicModelChoiceField(
        queryset=PrefixList.objects.all(),
        required=False,
        selector=True,
        label=_('Prefix List (in)'),
    )
    prefix_list_out = DynamicModelChoiceField(
        queryset=PrefixList.objects.all(),
        required=False,
        selector=True,
        label=_('Prefix List (out)'),
    )
    route_map_in = DynamicModelChoiceField(
        queryset=RouteMap.objects.all(),
        required=False,
        selector=True,
        label=_('Prefix List (in)'),
    )
    route_map_out = DynamicModelChoiceField(
        queryset=RouteMap.objects.all(),
        required=False,
        selector=True,
        label=_('Prefix List (out)'),
    )

    fieldsets = (
        FieldSet(
            TabbedGroups(
                FieldSet('peer', name=_('Peer')),
                FieldSet('peer_group', name=_('Peer Group')),
            ),
            name=_('Assigned Object'),
        ),
        FieldSet('address_family', 'enabled'),
        FieldSet(
            'route_map_in',
            'route_map_out',
            'prefix_list_in',
            'prefix_list_out',
            name=_('Filtering'),
        ),
        FieldSet('tenant_group', 'tenant', name=_('Tenancy')),
    )

    class Meta:
        model = BGPPeerAddressFamily
        fields = [
            'peer',
            'peer_group',
            'address_family',
            'enabled',
            'route_map_in',
            'route_map_out',
            'prefix_list_in',
            'prefix_list_out',
            'tenant_group',
            'tenant',
            'tags',
            'owner',
        ]

    def __init__(self, *args, **kwargs):

        # Initialize helper selectors
        instance = kwargs.get('instance')
        initial = kwargs.get('initial', {}).copy()
        if instance:
            if type(instance.assigned_object) is BGPPeer:
                initial['peer'] = instance.assigned_object
            elif type(instance.assigned_object) is BGPPeerTemplate:
                initial['peer_group'] = instance.assigned_object
        kwargs['initial'] = initial
        super().__init__(*args, **kwargs)

    def clean(self):
        super().clean()

        # Handle object assignment
        selected_objects = [
            field for field in ('peer', 'peer_group') if self.cleaned_data[field]
        ]
        if len(selected_objects) > 1:
            raise forms.ValidationError(
                {
                    selected_objects[1]: _(
                        "A BGP Peer Address Family can only be assigned to a single object."
                    )
                }
            )
        elif selected_objects:
            self.instance.assigned_object = self.cleaned_data[selected_objects[0]]
        else:
            raise ValidationError(
                _('A BGP Peer Address Family must specify an Peer or Peer Group.')
            )

    def save(self, *args, **kwargs):
        return super().save(*args, **kwargs)


class BFDProfileForm(TenancyForm, PrimaryModelForm):

    fieldsets = (
        FieldSet(
            'name',
            'description',
        ),
        FieldSet(
            'min_rx_int',
            'min_tx_int',
            'multiplier',
            'hold',
            name=_('BFD Parameters'),
        ),
        FieldSet('tenant_group', 'tenant', name=_('Tenancy')),
    )

    class Meta:
        model = BFDProfile
        fields = [
            'name',
            'min_tx_int',
            'min_rx_int',
            'multiplier',
            'hold',
            'description',
            'comments',
            'tenant_group',
            'tenant',
            'tags',
            'owner',
        ]



from netbox.api.viewsets import NetBoxModelViewSet
from netbox_routing import filtersets
from netbox_routing.api.serializers import StaticRouteSerializer
from netbox_routing.models import StaticRoute


class StaticRouteViewSet(NetBoxModelViewSet):
    queryset = StaticRoute.objects.all()
    serializer_class = StaticRouteSerializer
    filterset_class = filtersets.StaticRouteFilterSet



from netbox.plugins import PluginMenu

from .bfd import BFD_MENU
from .objects import OBJECT_MENU
from .ospf import MENUITEMS as OSPF_MENU
from .eigrp import eigrp
from .static import MENUITEMS as STATIC_MENU
from .bgp import BGP_MENU
from .community import COMMUNITY_MENU

__all__ = ('menu',)

menu = PluginMenu(
    label='Netbox Routing',
    groups=(
        ('Routing Objects', COMMUNITY_MENU + OBJECT_MENU),
        ('BFD', BFD_MENU),
        ('Static', STATIC_MENU),
        ('BGP', BGP_MENU),
        ('OSPF', OSPF_MENU),
        ('EIGRP', eigrp),
    ),
    icon_class='mdi mdi-router',
)



from netbox_routing.models import BGPRouter, BGPScope, BGPAddressFamily, BGPPeer
from netbox_routing.tests.base import ASNMixin, VRFMixin, AddressesMixin

__all__ = (
    'BGPRouterMixin',
    'BGPScopeMixin',
    'BGPAddressFamilyMixin',
    'BGPPeerMixin',
)


class BGPRouterMixin(ASNMixin):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.router = BGPRouter.objects.create(name='BGP Router', asn=cls.asn)


class BGPScopeMixin(VRFMixin, BGPRouterMixin):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.scope = BGPScope.objects.create(router=cls.router)


class BGPAddressFamilyMixin(BGPScopeMixin):
    afs = (
        'ipv4-unicast',
        'ipv6-unicast',
        'vpnv4-unicast',
        'vpnv6-unicast',
        'l2vpn-vpls',
        'l2vpn-evpn',
        'ipv4-multicast',
        'ipv6-multicast',
    )

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        for af in cls.afs:
            BGPAddressFamily.objects.create(scope=cls.scope, address_family=af)

        cls.af = BGPAddressFamily.objects.first()
        cls.address_family = BGPAddressFamily.objects.first()
        cls.address_families = BGPAddressFamily.objects.all()


class BGPPeerMixin(BGPScopeMixin, AddressesMixin):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.peer = BGPPeer.objects.create(
            name='BGP Peer', scope=cls.scope, peer=cls.peer_address
        )



# Generated by Django 4.1.7 on 2023-05-18 13:45

from django.db import migrations, models
import django.db.models.functions.text


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0002_netboxmodel_updates'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='prefixlist',
            options={'ordering': ['name']},
        ),
        migrations.AlterModelOptions(
            name='prefixlistentry',
            options={'ordering': ['prefix_list', 'sequence']},
        ),
        migrations.AlterModelOptions(
            name='routemap',
            options={'ordering': ['name']},
        ),
        migrations.AlterModelOptions(
            name='routemapentry',
            options={'ordering': ['route_map', 'sequence']},
        ),
        migrations.AlterModelOptions(
            name='staticroute',
            options={'ordering': ['vrf', 'prefix', 'metric']},
        ),
        migrations.AddConstraint(
            model_name='prefixlist',
            constraint=models.UniqueConstraint(
                django.db.models.functions.text.Lower('name'),
                name='netbox_routing_prefixlist_unique_name',
                violation_error_message='Name must be unique.',
            ),
        ),
        migrations.AddConstraint(
            model_name='prefixlistentry',
            constraint=models.UniqueConstraint(
                models.F('prefix_list'),
                models.F('sequence'),
                name='netbox_routing_prefixlistentry_unique_prefixlist_sequence',
                violation_error_message='Prefix List sequence must be unique.',
            ),
        ),
        migrations.AddConstraint(
            model_name='routemap',
            constraint=models.UniqueConstraint(
                django.db.models.functions.text.Lower('name'),
                name='netbox_routing_routemap_unique_name',
                violation_error_message='Name must be unique.',
            ),
        ),
        migrations.AddConstraint(
            model_name='routemapentry',
            constraint=models.UniqueConstraint(
                models.F('route_map'),
                models.F('sequence'),
                name='netbox_routing_routemapentry_unique_routemap_sequence',
                violation_error_message='Route Map sequence must be unique.',
            ),
        ),
        migrations.AddConstraint(
            model_name='staticroute',
            constraint=models.UniqueConstraint(
                models.F('vrf'),
                models.F('prefix'),
                models.F('next_hop'),
                name='netbox_routing_staticroute_unique_vrf_prefix_nexthop',
                violation_error_message='VRF, Prefix and Next Hop must be unique.',
            ),
        ),
    ]



# from django.urls import reverse
# from netaddr.ip import IPAddress
# from rest_framework import status

# from dcim.models import Interface
# from ipam.models import VRF
from utilities.testing import APIViewTestCases  # , APITestCase, create_test_device

# from netbox_routing.models import EIGRPRouter, EIGRPAddressFamily, EIGRPNetwork, EIGRPInterface
# from netbox_routing.tests.base import IPAddressFieldMixin

__all__ = (
    'EIGRPRouterTestCase',
    'EIGRPAddressFamilyTestCase',
    'EIGRPNetworkTestCase',
    'EIGRPInterfaceTestCase',
)


class EIGRPRouterTestCase(APIViewTestCases):
    pass


class EIGRPAddressFamilyTestCase(APIViewTestCases):
    pass


class EIGRPNetworkTestCase(APIViewTestCases):
    pass


class EIGRPInterfaceTestCase(APIViewTestCases):
    pass



# Generated by Django 5.1.8 on 2025-04-21 21:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0012_osfp_instance_vrf'),
    ]

    operations = [
        migrations.AddField(
            model_name='staticroute',
            name='tag',
            field=models.IntegerField(blank=True, null=True),
        ),
    ]



from rest_framework import serializers

from dcim.api.serializers_.device_components import InterfaceSerializer
from dcim.api.serializers_.devices import DeviceSerializer
from ipam.api.serializers_.vrfs import VRFSerializer
from netbox.api.serializers import NetBoxModelSerializer
from netbox_routing.models import OSPFInstance, OSPFArea, OSPFInterface

__all__ = (
    'OSPFInstanceSerializer',
    'OSPFAreaSerializer',
    'OSPFInterfaceSerializer',
)


class OSPFInstanceSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:ospfinstance-detail'
    )
    device = DeviceSerializer(nested=True)
    vrf = VRFSerializer(nested=True)

    class Meta:
        model = OSPFInstance
        fields = (
            'url',
            'id',
            'display',
            'name',
            'router_id',
            'process_id',
            'device',
            'vrf',
            'description',
            'comments',
            'custom_fields',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'name',
            'router_id',
            'process_id',
            'device',
            'vrf',
        )


class OSPFAreaSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:ospfarea-detail'
    )

    class Meta:
        model = OSPFArea
        fields = (
            'url',
            'id',
            'display',
            'area_id',
            'area_type',
            'description',
            'comments',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'area_id',
            'area_type',
        )


class OSPFInterfaceSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:ospfarea-detail'
    )
    instance = OSPFInstanceSerializer(nested=True)
    area = OSPFAreaSerializer(nested=True)
    interface = InterfaceSerializer(nested=True)

    class Meta:
        model = OSPFInterface
        fields = (
            'url',
            'id',
            'display',
            'instance',
            'area',
            'interface',
            'passive',
            'priority',
            'bfd',
            'authentication',
            'passphrase',
            'description',
            'comments',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'instance',
            'area',
            'interface',
            'passive',
        )



from utilities.choices import ChoiceSet

__all__ = (
    'BGPAdditionalPathSelectChoices',
    'BGPAddressFamilyChoices',
    'BGPBestPathASPathChoices',
    'BGPSettingChoices',
    'BFDChoices',
    'BGPStatusChoices',
)


class BGPSettingChoices(ChoiceSet):
    SUMMARY = 'auto_summary'

    RID = 'router_id'
    ADDPATH_INSTALL = 'additional_paths_install'
    ADDPATH_RECEIVE = 'additional_paths_receive'
    ADDPATH_SEND = 'additional_paths_send'
    ASDOT = 'asdot'
    GR = 'graceful_restart'
    DEFAULT_ORIGINATE = 'default_information_originate'
    DEFAULT_METRIC = 'default_metric'
    DISTANCE_EBGP = 'distance_ebgp'
    DISTANCE_IBGP = 'distance_ibgp'
    DISTANCE_EMBGP = 'distance_embgp'
    DISTANCE_IMBGP = 'distance_imbgp'
    MAX_PATHS = 'paths_maximum'
    MAX_PATHS_SECONDARY = 'paths_maximum_secondary'
    KEEPALIVE = 'timers_keepalive'
    HOLD = 'timers_hold'

    CHOICES = [
        (RID, 'Router ID'),
        (SUMMARY, 'Auto summary'),
        (ADDPATH_INSTALL, 'Additional Paths (install)'),
        (ADDPATH_RECEIVE, 'Additional Paths (receive)'),
        (ADDPATH_SEND, 'Additional Paths (send)'),
        (ASDOT, 'AS Dot Notation'),
        (GR, 'Graceful Restart'),
        (DEFAULT_ORIGINATE, 'Default Originate'),
        (DEFAULT_METRIC, 'Default Metric'),
        (DISTANCE_EBGP, 'eBGP Distance'),
        (DISTANCE_IBGP, 'iBGP Distance'),
        (DISTANCE_EMBGP, 'MP eBGP Distance'),
        (DISTANCE_IMBGP, 'MP iBGP Distance'),
        (MAX_PATHS, 'Maximum Paths'),
        (MAX_PATHS_SECONDARY, 'Maximum Paths (secondary)'),
        (KEEPALIVE, 'Keep Alive'),
        (HOLD, 'Hold Time'),
    ]

    FIELD_TYPES = {
        RID: 'ipaddr',
        SUMMARY: 'boolean',
        ADDPATH_INSTALL: 'integer',
        ADDPATH_RECEIVE: 'integer',
        ADDPATH_SEND: 'integer',
        ASDOT: 'boolean',
        GR: 'boolean',
        DEFAULT_ORIGINATE: 'boolean',
        DEFAULT_METRIC: 'integer',
        DISTANCE_EBGP: 'integer',
        DISTANCE_IBGP: 'integer',
        DISTANCE_EMBGP: 'integer',
        DISTANCE_IMBGP: 'integer',
        MAX_PATHS: 'integer',
        MAX_PATHS_SECONDARY: 'integer',
        KEEPALIVE: 'integer',
        HOLD: 'integer',
    }


class BGPAdditionalPathSelectChoices(ChoiceSet):
    ALL = 'all'
    BACKUP = 'backup'
    BEST_EXTERNAL = 'best-external'
    GROUP_BEST = 'group-best'

    CHOICES = [
        (ALL, 'All'),
        (BACKUP, 'Backup'),
        (BEST_EXTERNAL, 'Best External'),
        (GROUP_BEST, 'Group Best'),
    ]


class BFDChoices(ChoiceSet):
    SINGLEHOP = 'singlehop'
    MULTIHOP = 'multihop'

    CHOICES = [(SINGLEHOP, 'Single-Hop'), (MULTIHOP, 'Multi-Hop')]


class BGPBestPathASPathChoices(ChoiceSet):
    IGNORE = 'ignore'
    MULTIPATH = 'multipath-relax'

    CHOICES = [(IGNORE, 'Ignore'), (MULTIPATH, 'Multipath Relax Comparison')]


class BGPAddressFamilyChoices(ChoiceSet):
    IPV4_UNICAST = 'ipv4-unicast'
    IPV6_UNICAST = 'ipv6-unicast'
    VPNV4_UNICAST = 'vpnv4-unicast'
    VPNV6_UNICAST = 'vpnv6-unicast'
    IPV4_MULTICAST = 'ipv4-multicast'
    IPV6_MULTICAST = 'ipv6-multicast'
    VPNV4_MULTICAST = 'vpnv4-multicast'
    VPNV6_MULTICAST = 'vpnv6-multicast'
    IPV4_FLOWSPEC = 'ipv4-flowspec'
    IPV6_FLOWSPEC = 'ipv6-flowspec'
    VPNV4_FLOWSPEC = 'vpnv4-flowspec'
    VPNV6_FLOWSPEC = 'vpnv6-flowspec'
    NSAP = 'nsap'
    L2VPNVPLS = 'l2vpn-vpls'
    L2VPSEVPN = 'l2vpn-evpn'
    LINKSTATE = 'link-state'
    RTFILTER_UNICAST = 'rtfilter-unicast'

    CHOICES = [
        (IPV4_UNICAST, 'IPv4 Unicast'),
        (IPV6_UNICAST, 'IPv6 Unicast'),
        (VPNV4_UNICAST, 'VPNv4 Unicast'),
        (VPNV6_UNICAST, 'VPNv6 Unicast'),
        (IPV4_MULTICAST, 'IPv4 Multicast'),
        (IPV6_MULTICAST, 'IPv6 Multicast'),
        (VPNV4_UNICAST, 'VPNv4 Multicast'),
        (VPNV6_MULTICAST, 'VPNv6 Multicast'),
        (IPV4_FLOWSPEC, 'IPv4 Flowspec'),
        (IPV6_FLOWSPEC, 'IPv6 Flowspec'),
        (VPNV4_FLOWSPEC, 'VPNv4 Flowspec'),
        (VPNV6_FLOWSPEC, 'VPNv6 Flowspec'),
        (NSAP, 'NSAP'),
        (L2VPNVPLS, 'L2VPN VPLS'),
        (L2VPSEVPN, 'L2VPN EVPN'),
        (LINKSTATE, 'LINK-STATE'),
        (RTFILTER_UNICAST, 'RTFILTER'),
    ]


class BGPStatusChoices(ChoiceSet):
    ACTIVE = 'active'
    PLANNED = 'planned'
    OFFLINE = 'offline'
    FAILED = 'failed'

    CHOICES = [
        (ACTIVE, 'Active'),
        (PLANNED, 'Planned'),
        (OFFLINE, 'Offline'),
        (FAILED, 'Failed'),
    ]



from .filtersets import *
from .bulk_edit import *
from .bulk_import import *
from .objects import (
    PrefixListForm,
    PrefixListEntryForm,
    RouteMapForm,
    RouteMapEntryForm,
)
from netbox_routing.forms.model_objects.ospf import *
from netbox_routing.forms.model_objects.bgp import *
from netbox_routing.forms.model_objects.static import StaticRouteForm
from netbox_routing.forms.model_objects.eigrp import *

__all__ = (
    # Static Routes
    'StaticRouteForm',
    'StaticRouteFilterForm',
    # OSPF
    'OSPFAreaForm',
    'OSPFAreaBulkEditForm',
    'OSPFAreaImportForm',
    'OSPFAreaFilterForm',
    'OSPFInstanceForm',
    'OSPFInstanceBulkEditForm',
    'OSPFInstanceFilterForm',
    'OSPFInstanceImportForm',
    'OSPFInterfaceForm',
    'OSPFInterfaceFilterForm',
    'OSPFInterfaceBulkEditForm',
    'OSPFInterfaceImportForm',
    # EIGRP
    'EIGRPRouterForm',
    'EIGRPRouterBulkEditForm',
    'EIGRPRouterFilterForm',
    'EIGRPRouterImportForm',
    'EIGRPAddressFamilyForm',
    'EIGRPAddressFamilyBulkEditForm',
    'EIGRPAddressFamilyFilterForm',
    'EIGRPAddressFamilyImportForm',
    'EIGRPNetworkForm',
    'EIGRPNetworkBulkEditForm',
    'EIGRPNetworkFilterForm',
    'EIGRPNetworkImportForm',
    'EIGRPInterfaceForm',
    'EIGRPInterfaceBulkEditForm',
    'EIGRPInterfaceFilterForm',
    'EIGRPInterfaceImportForm',
    # BGP
    'BGPSettingForm',
    'BGPRouterForm',
    'BGPScopeForm',
    'BGPAddressFamilyForm',
    'BGPRouterFilterForm',
    'BGPScopeFilterForm',
    'BGPAddressFamilyFilterForm',
    'BGPSettingFilterForm',
    # BFD
    'BFDProfileForm',
    'BFDProfileBulkEditForm',
    'BFDProfileFilterForm',
    # Objects
    'PrefixListForm',
    'PrefixListEntryForm',
    'RouteMapForm',
    'RouteMapEntryForm',
    'PrefixListFilterForm',
    'PrefixListEntryFilterForm',
    'RouteMapFilterForm',
    'RouteMapEntryFilterForm',
)



from netbox.plugins import PluginMenuButton, PluginMenuItem

__all__ = ('MENUITEMS',)

COL_ADD = 'mdi mdi-plus'
COL_IMPORT = 'mdi mdi-upload'

static = PluginMenuItem(
    link='plugins:netbox_routing:staticroute_list',
    link_text='Static Routes',
    permissions=['netbox_routing.view_staticroute'],
    buttons=(
        PluginMenuButton(
            link='plugins:netbox_routing:staticroute_add',
            title='Add',
            icon_class=COL_ADD,
            permissions=['netbox_routing.add_staticroute'],
        ),
        PluginMenuButton(
            link='plugins:netbox_routing:staticroute_bulk_import',
            title='Import',
            icon_class=COL_IMPORT,
            permissions=['netbox_routing.import_staticroute'],
        ),
    ),
)

MENUITEMS = (static,)



from .bgp import *
from .community import *
from .eigrp import *
from .objects import *
from .ospf import OSPFArea, OSPFInstance, OSPFInterface
from .static import StaticRoute

__all__ = (
    # BGP
    'BGPRouter',
    'BGPScope',
    'BGPAddressFamily',
    'BGPPeerTemplate',
    'BGPPolicyTemplate',
    'BGPSessionTemplate',
    'BGPSetting',
    'BGPPeer',
    'BGPPeerAddressFamily',
    # Communities
    'Community',
    'CommunityList',
    'CommunityListEntry',
    # EIGRP
    'EIGRPRouter',
    'EIGRPAddressFamily',
    'EIGRPNetwork',
    'EIGRPInterface',
    # OSPF
    'OSPFArea',
    'OSPFInstance',
    'OSPFInterface',
    # Objects -> AS Path
    'ASPath',
    'ASPathEntry',
    # Objects -> Prefix Lists
    'PrefixList',
    'PrefixListEntry',
    'CustomPrefix',
    # Objects -> Route Maps
    'RouteMap',
    'RouteMapEntry',
    # Static Routing
    'StaticRoute',
)



from django import forms
from django.utils.translation import gettext as _

from ipam.models import ASN, IPAddress, VRF
from netbox.forms import PrimaryModelBulkEditForm
from netbox_routing.forms.bulk_edit.base import EnableMixin, TenantBulkEditMixin
from netbox_routing.models import RouteMap, PrefixList
from utilities.forms.fields import (
    DynamicModelChoiceField,
    DynamicModelMultipleChoiceField,
)
from utilities.forms.rendering import FieldSet

from netbox_routing.choices.bgp import *
from netbox_routing.models.bgp import *

__all__ = (
    'BGPSettingBulkEditForm',
    'BGPRouterBulkEditForm',
    'BGPScopeBulkEditForm',
    'BGPAddressFamilyBulkEditForm',
    'BGPPeerTemplateBulkEditForm',
    'BGPPolicyTemplateBulkEditForm',
    'BGPSessionTemplateBulkEditForm',
    'BGPPeerBulkEditForm',
    'BGPPeerAddressFamilyBulkEditForm',
    'BFDProfileBulkEditForm',
)


class BGPPolicyMixin:
    prefix_list_in = DynamicModelChoiceField(
        label=_('Prefix List (in)'),
        queryset=PrefixList.objects.all(),
        required=False,
        selector=True,
    )
    prefix_list_out = DynamicModelChoiceField(
        label=_('Prefix List (out)'),
        queryset=PrefixList.objects.all(),
        required=False,
        selector=True,
    )
    routemap_in = DynamicModelChoiceField(
        label=_('Route Map (in)'),
        queryset=RouteMap.objects.all(),
        required=False,
        selector=True,
    )
    routemap_out = DynamicModelChoiceField(
        label=_('Route Map (out)'),
        queryset=RouteMap.objects.all(),
        required=False,
        selector=True,
    )


class BGPSessionMixin:
    remote_as = DynamicModelChoiceField(
        label=_('Remote AS'), queryset=ASN.objects.all(), required=False, selector=True
    )
    local_as = DynamicModelChoiceField(
        label=_('Local AS'), queryset=ASN.objects.all(), required=False, selector=True
    )


class BGPSettingBulkEditForm(TenantBulkEditMixin, PrimaryModelBulkEditForm):
    model = BGPSetting
    fieldsets = (
        FieldSet(
            'description',
        ),
    )
    nullable_fields = ('description',)


class BGPRouterBulkEditForm(TenantBulkEditMixin, PrimaryModelBulkEditForm):
    asn = DynamicModelChoiceField(
        label=_('ASN'), queryset=ASN.objects.all(), required=False, selector=True
    )
    peer_templates = DynamicModelMultipleChoiceField(
        label=_('Assigned Peer Template'),
        queryset=BGPPeerTemplate.objects.all(),
        required=False,
        selector=True,
    )
    policy_templates = DynamicModelMultipleChoiceField(
        label=_('Assigned Policy Template'),
        queryset=BGPPolicyTemplate.objects.all(),
        required=False,
        selector=True,
    )
    session_templates = DynamicModelMultipleChoiceField(
        label=_('Assigned Session Template'),
        queryset=BGPSessionTemplate.objects.all(),
        required=False,
        selector=True,
    )
    model = BGPRouter
    fieldsets = (
        FieldSet(
            'asn',
            'policy_templates',
            'session_templates',
            'peer_templates',
            'description',
        ),
        FieldSet('tenant', name=_('Tenancy')),
    )
    nullable_fields = (
        'policy_templates',
        'session_templates',
        'peer_templates',
        'description',
        'tenant',
    )


class BGPScopeBulkEditForm(TenantBulkEditMixin, PrimaryModelBulkEditForm):
    router = DynamicModelChoiceField(
        label=_('Router'),
        queryset=BGPRouter.objects.all(),
        required=False,
        selector=True,
    )
    vrf = DynamicModelChoiceField(
        label=_('VRF'), queryset=VRF.objects.all(), required=False, selector=True
    )
    model = BGPScope
    fieldsets = (
        FieldSet(
            'router',
            'vrf',
            'description',
        ),
        FieldSet('tenant', name=_('Tenancy')),
    )
    nullable_fields = (
        'router',
        'vrf',
        'description',
        'tenant',
    )


class BGPAddressFamilyBulkEditForm(TenantBulkEditMixin, PrimaryModelBulkEditForm):
    scope = DynamicModelChoiceField(
        label=_('Scope'), queryset=BGPScope.objects.all(), required=False, selector=True
    )
    address_family = forms.ChoiceField(
        label=_('Status'),
        choices=BGPAddressFamilyChoices,
        required=False,
    )
    model = BGPAddressFamily
    fieldsets = (
        FieldSet(
            'scope',
            'address_family',
            'description',
        ),
        FieldSet('tenant', name=_('Tenancy')),
    )
    nullable_fields = (
        'scope',
        'description',
        'tenant',
    )


class BGPPeerTemplateBulkEditForm(
    EnableMixin, TenantBulkEditMixin, PrimaryModelBulkEditForm
):
    asn = DynamicModelChoiceField(
        label=_('ASN'), queryset=ASN.objects.all(), required=False, selector=True
    )

    model = BGPPeerTemplate
    fieldsets = (
        FieldSet(
            'enabled',
            'remote_as',
        ),
        FieldSet('tenant', name=_('Tenancy')),
    )
    nullable_fields = (
        'remote_as',
        'enabled',
        'tenant',
    )


class BGPPolicyTemplateBulkEditForm(
    EnableMixin, BGPPolicyMixin, TenantBulkEditMixin, PrimaryModelBulkEditForm
):
    parents = DynamicModelMultipleChoiceField(
        label=_('Parent Policy Templates'),
        queryset=BGPPolicyTemplate.objects.all(),
        required=False,
        selector=True,
    )
    model = BGPPolicyTemplate
    fieldsets = (
        FieldSet(
            'enabled',
            'parents',
        ),
        FieldSet(
            'prefixlist_in',
            'prefixlist_out',
            'routemap_in',
            'routemap_out',
        ),
        FieldSet('tenant', name=_('Tenancy')),
    )
    nullable_fields = (
        'enabled',
        'parents',
        'prefixlist_in',
        'prefixlist_out',
        'routemap_in',
        'routemap_out',
        'tenant',
    )


class BGPSessionTemplateBulkEditForm(
    EnableMixin, BGPSessionMixin, TenantBulkEditMixin, PrimaryModelBulkEditForm
):
    parent = DynamicModelChoiceField(
        label=_('Parent'),
        queryset=BGPSessionTemplate.objects.all(),
        required=False,
        selector=True,
    )
    bfd = forms.BooleanField(label=_('BFD Enabled'), required=False)
    password = forms.CharField(label=_('Password'), required=False)

    model = BGPSessionTemplate
    fieldsets = (
        FieldSet(
            'parent',
            'enabled',
            'remote_as',
            'local_as',
            'bfd',
            'ttl',
            'password',
        ),
        FieldSet('tenant', name=_('Tenancy')),
    )
    nullable_fields = (
        'parent',
        'enabled',
        'remote_as',
        'local_as',
        'bfd',
        'ttl',
        'password',
        'tenant',
    )


class BGPPeerBulkEditForm(
    EnableMixin, BGPSessionMixin, TenantBulkEditMixin, PrimaryModelBulkEditForm
):
    scope = DynamicModelChoiceField(
        label=_('Scope'), queryset=BGPScope.objects.all(), required=False, selector=True
    )
    peer = DynamicModelChoiceField(
        label=_('Peer'), queryset=IPAddress.objects.all(), required=False, selector=True
    )
    source = DynamicModelChoiceField(
        label=_('Source'),
        queryset=IPAddress.objects.all(),
        required=False,
        selector=True,
    )
    peer_group = DynamicModelChoiceField(
        label=_('Peer Group'),
        queryset=BGPPeerTemplate.objects.all(),
        required=False,
        selector=True,
    )
    peer_session = DynamicModelChoiceField(
        label=_('Peer Session'),
        queryset=BGPSessionTemplate.objects.all(),
        required=False,
        selector=True,
    )
    bfd = forms.BooleanField(label=_('BFD Enabled'), required=False)
    password = forms.CharField(label=_('Password'), required=False)
    model = BGPPeer
    fieldsets = (
        FieldSet(
            'scope',
            'peer',
            'status',
            'enabled',
            'source',
            'remote_as',
            'local_as',
        ),
        FieldSet(
            'peer_group',
            'peer_session',
            'bfd',
            'ttl',
            'password',
        ),
        FieldSet('tenant', name=_('Tenancy')),
    )
    nullable_fields = (
        'source',
        'peer_group',
        'peer_session',
        'local_as',
        'status',
        'ttl',
        'enabled',
        'password',
        'tenant',
    )


class BGPPeerAddressFamilyBulkEditForm(
    EnableMixin, BGPPolicyMixin, TenantBulkEditMixin, PrimaryModelBulkEditForm
):
    peer_policy = DynamicModelChoiceField(
        label=_('Peer Policy'),
        queryset=BGPPolicyTemplate.objects.all(),
        required=False,
        selector=True,
    )
    model = BGPPeerAddressFamily
    fieldsets = (
        FieldSet(
            'enabled',
            'peer_policy',
        ),
        FieldSet(
            'prefixlist_out',
            'prefixlist_in',
            'routemap_out',
            'routemap_in',
        ),
        FieldSet('tenant', name=_('Tenancy')),
    )
    nullable_fields = (
        'peer_policy',
        'prefixlist_out',
        'prefixlist_in',
        'routemap_out',
        'routemap_in',
        'tenant',
    )


class BFDProfileBulkEditForm(TenantBulkEditMixin, PrimaryModelBulkEditForm):
    model = BFDProfile
    fieldsets = (
        FieldSet(
            'description',
        ),
        FieldSet(
            'min_tx_int',
            'min_rx_int',
            'multiplier',
            'hold',
        ),
        FieldSet('tenant', name=_('Tenancy')),
    )
    nullable_fields = (
        'hold',
        'description',
        'tenant',
    )



import netaddr
from utilities.testing import ViewTestCases
from ipam.models import IPAddress, VRF, ASN

from netbox_routing.models.bgp import *
from netbox_routing.tests.base import *
from netbox_routing.tests.bgp.base import *

__all__ = (
    'BGPRouterTestCase',
    'BGPScopeTestCase',
)


class BGPAutomatedFormDataCreationMixin(AutomatedModelCreationMixin):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.form_data = {
            'name': '{cls.model._meta.verbose_name}: 4',
        }


class BGPRouterTestCase(
    ASNMixin,
    BulkEditMixin,
    PluginBaseURLMixin,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = BGPRouter

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        items = []
        name = 'BGP Router'
        for seq in range(1, 4):
            asn = ASN.objects.all()[seq - 1]
            items.append(
                cls.model(
                    name=f'{name} {seq}',
                    asn=asn,
                ),
            )
        cls.model.objects.bulk_create(items)

        cls.form_data = {'name': f'{name} 4', 'asn': ASN.objects.last().pk}


class BGPScopeTestCase(
    BGPRouterMixin,
    BulkEditMixin,
    PluginBaseURLMixin,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = BGPScope

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        items = []
        for seq in range(1, 4):
            vrf = VRF.objects.create(name=f'VRF {seq}', rd=f'64512:{seq}')
            items.append(
                cls.model(
                    router=cls.router,
                    vrf=vrf,
                ),
            )
        cls.model.objects.bulk_create(items)

        cls.form_data = {
            'router': cls.router.pk,
        }


class BGPAddressFamilyTestCase(
    BGPScopeMixin,
    BulkEditMixin,
    PluginBaseURLMixin,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = BGPAddressFamily

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        items = []
        for seq in ('ipv4-unicast', 'ipv6-unicast', 'vpnv4-unicast', 'vpnv6-unicast'):
            items.append(
                cls.model(
                    scope=cls.scope,
                    address_family=seq,
                ),
            )
        cls.model.objects.bulk_create(items)

        cls.form_data = {
            'scope': cls.scope.pk,
            'address_family': 'l2vpn-evpn',
        }


class BGPPeerTestCase(
    BGPScopeMixin,
    BulkEditMixin,
    PluginBaseURLMixin,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = BGPPeer

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        items = []
        for seq in (
            '10.0.0.1/24',
            '10.0.0.2/24',
            '10.0.0.3/24',
            '10.0.0.4/24',
            '10.0.0.5/24',
            '10.0.0.6/24',
        ):
            peer = IPAddress.objects.create(address=netaddr.IPNetwork(seq))
            items.append(
                cls.model(
                    name=f'BGP Peer: {seq}',
                    scope=cls.scope,
                    peer=peer,
                ),
            )
        cls.model.objects.bulk_create(items)

        peer = IPAddress.objects.create(address=netaddr.IPNetwork('10.0.0.7/24'))
        cls.form_data = {
            'name': 'BGP Peer: 10.0.0.7/24',
            'scope': cls.scope.pk,
            'peer': peer.pk,
        }


class BGPPeerAddressFamilyTestCase(
    AutomatedModelCreationMixin,
    BGPPeerMixin,
    BGPAddressFamilyMixin,
    BulkEditMixin,
    PluginBaseURLMixin,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = BGPPeerAddressFamily
    validation_excluded_fields = ('peer',)
    routing_required_fields = (
        ('assigned_object', 'peer'),
        ('address_family', 'address_families'),
        'enabled',
    )

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        af = BGPAddressFamily.objects.last()
        cls.form_data = {
            'peer': cls.peer.pk,
            'address_family': af.pk,
        }


class BGPPeerTemplateTestCase(
    AutomatedModelCreationMixin,
    BGPAddressFamilyMixin,
    BulkEditMixin,
    PluginBaseURLMixin,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = BGPPeerTemplate
    routing_required_fields = ('name',)

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        cls.form_data = {'name': f'{cls.model._meta.verbose_name} 5'}


class BGPPolicyTemplateTestCase(
    AutomatedModelCreationMixin,
    BGPAddressFamilyMixin,
    BulkEditMixin,
    PluginBaseURLMixin,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = BGPPolicyTemplate
    routing_required_fields = ('name',)

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        cls.form_data = {'name': f'{cls.model._meta.verbose_name} 5'}


class BGPSessionTemplateTestCase(
    AutomatedModelCreationMixin,
    BGPAddressFamilyMixin,
    BulkEditMixin,
    PluginBaseURLMixin,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = BGPSessionTemplate
    routing_required_fields = ('name',)

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        cls.form_data = {'name': f'{cls.model._meta.verbose_name} 5'}


class BFDProfileTestCase(
    BulkEditMixin,
    PluginBaseURLMixin,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = BFDProfile

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        profiles = []
        for i in range(1, 4):
            profiles.append(
                BFDProfile(
                    name=f'Test BFD Profile {i}',
                    min_rx_int=60,
                    min_tx_int=60,
                    multiplier=3,
                )
            )
        BFDProfile.objects.bulk_create(profiles)

        cls.form_data = {
            'name': f'{cls.model._meta.verbose_name} 5',
            'min_rx_int': 60,
            'min_tx_int': 60,
            'multiplier': 3,
        }



from netbox.views.generic import (
    ObjectListView,
    ObjectEditView,
    ObjectView,
    ObjectDeleteView,
    ObjectChildrenView,
    BulkImportView,
    BulkEditView,
    BulkDeleteView,
)
from netbox_routing.filtersets.eigrp import *
from netbox_routing.forms import *
from netbox_routing.tables.eigrp import *
from utilities.views import register_model_view, ViewTab

from netbox_routing.models import *

__all__ = (
    'EIGRPRouterListView',
    'EIGRPRouterView',
    'EIGRPRouterAddressFamiliesView',
    'EIGRPRouterInterfacesView',
    'EIGRPRouterNetworksView',
    'EIGRPRouterEditView',
    'EIGRPRouterImportView',
    'EIGRPRouterBulkEditView',
    'EIGRPRouterDeleteView',
    'EIGRPRouterBulkDeleteView',
    'EIGRPAddressFamilyListView',
    'EIGRPAddressFamilyView',
    'EIGRPAddressFamilyInterfacesView',
    'EIGRPAddressFamilyNetworksView',
    'EIGRPAddressFamilyEditView',
    'EIGRPAddressFamilyBulkEditView',
    'EIGRPAddressFamilyDeleteView',
    'EIGRPAddressFamilyBulkDeleteView',
    'EIGRPNetworkListView',
    'EIGRPNetworkView',
    'EIGRPNetworkEditView',
    'EIGRPNetworkBulkEditView',
    'EIGRPNetworkDeleteView',
    'EIGRPNetworkBulkDeleteView',
    'EIGRPInterfaceListView',
    'EIGRPInterfaceView',
    'EIGRPInterfaceEditView',
    'EIGRPInterfaceBulkEditView',
    'EIGRPInterfaceDeleteView',
    'EIGRPInterfaceBulkDeleteView',
)


#
# Instance
#


@register_model_view(EIGRPRouter, name='list', path='', detail=False)
class EIGRPRouterListView(ObjectListView):
    queryset = EIGRPRouter.objects.all()
    table = EIGRPRouterTable
    filterset = EIGRPRouterFilterSet
    filterset_form = EIGRPRouterFilterForm


@register_model_view(EIGRPRouter)
class EIGRPRouterView(ObjectView):
    queryset = EIGRPRouter.objects.all()
    template_name = 'netbox_routing/eigrprouter.html'


@register_model_view(EIGRPRouter, name='address_families')
class EIGRPRouterAddressFamiliesView(ObjectChildrenView):
    queryset = EIGRPRouter.objects.all()
    child_model = EIGRPAddressFamily
    table = EIGRPAddressFamilyTable
    filterset = EIGRPAddressFamilyFilterSet
    tab = ViewTab(
        label='Address Families',
        badge=lambda obj: EIGRPAddressFamily.objects.filter(router=obj).count(),
        hide_if_empty=False,
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(router=parent)


@register_model_view(EIGRPRouter, name='interfaces')
class EIGRPRouterInterfacesView(ObjectChildrenView):
    queryset = EIGRPRouter.objects.all()
    child_model = EIGRPInterface
    table = EIGRPInterfaceTable
    filterset = EIGRPInterfaceFilterSet
    tab = ViewTab(
        label='Interfaces',
        badge=lambda obj: EIGRPInterface.objects.filter(router=obj).count(),
        hide_if_empty=False,
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(router=parent)


@register_model_view(EIGRPRouter, name='networks')
class EIGRPRouterNetworksView(ObjectChildrenView):
    queryset = EIGRPRouter.objects.all()
    child_model = EIGRPNetwork
    table = EIGRPNetworkTable
    filterset = EIGRPNetworkFilterSet
    tab = ViewTab(
        label='Networks',
        badge=lambda obj: EIGRPNetwork.objects.filter(router=obj).count(),
        hide_if_empty=False,
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(router=parent)


@register_model_view(EIGRPRouter, name='add', detail=False)
@register_model_view(EIGRPRouter, name='edit')
class EIGRPRouterEditView(ObjectEditView):
    queryset = EIGRPRouter.objects.all()
    form = EIGRPRouterForm


@register_model_view(EIGRPRouter, name='delete')
class EIGRPRouterDeleteView(ObjectDeleteView):
    queryset = EIGRPRouter.objects.all()


@register_model_view(EIGRPRouter, name='bulk_edit', detail=False)
class EIGRPRouterBulkEditView(BulkEditView):
    queryset = EIGRPRouter.objects.all()
    filterset = EIGRPRouterFilterSet
    table = EIGRPRouterTable
    form = EIGRPRouterBulkEditForm


@register_model_view(EIGRPRouter, name='bulk_delete', detail=False)
class EIGRPRouterBulkDeleteView(BulkDeleteView):
    queryset = EIGRPRouter.objects.all()
    filterset = EIGRPRouterFilterSet
    table = EIGRPRouterTable


@register_model_view(EIGRPRouter, name='bulk_import', detail=False)
class EIGRPRouterImportView(BulkImportView):
    queryset = EIGRPRouter.objects.all()
    model_form = EIGRPRouterImportForm


#
# Address Family
#


@register_model_view(EIGRPAddressFamily, name='list', path='', detail=False)
class EIGRPAddressFamilyListView(ObjectListView):
    queryset = EIGRPAddressFamily.objects.all()
    table = EIGRPAddressFamilyTable
    filterset = EIGRPAddressFamilyFilterSet
    filterset_form = EIGRPAddressFamilyFilterForm


@register_model_view(EIGRPAddressFamily)
class EIGRPAddressFamilyView(ObjectView):
    queryset = EIGRPAddressFamily.objects.all()
    template_name = 'netbox_routing/eigrpaddressfamily.html'


@register_model_view(EIGRPAddressFamily, name='interfaces')
class EIGRPAddressFamilyInterfacesView(ObjectChildrenView):
    # template_name = 'netbox_routing/object_children.html'
    queryset = EIGRPAddressFamily.objects.all()
    child_model = EIGRPInterface
    table = EIGRPInterfaceTable
    filterset = EIGRPInterfaceFilterSet
    tab = ViewTab(
        label='Interfaces',
        badge=lambda obj: EIGRPInterface.objects.filter(address_family=obj).count(),
    )


@register_model_view(EIGRPAddressFamily, name='networks')
class EIGRPAddressFamilyNetworksView(ObjectChildrenView):
    # template_name = 'netbox_routing/object_children.html'
    queryset = EIGRPAddressFamily.objects.all()
    child_model = EIGRPNetwork
    table = EIGRPNetworkTable
    filterset = EIGRPNetworkFilterSet
    tab = ViewTab(
        label='Interfaces',
        badge=lambda obj: EIGRPInterface.objects.filter(address_family=obj).count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(address_famiily=parent)


@register_model_view(EIGRPAddressFamily, name='add', detail=False)
@register_model_view(EIGRPAddressFamily, name='edit')
class EIGRPAddressFamilyEditView(ObjectEditView):
    queryset = EIGRPAddressFamily.objects.all()
    form = EIGRPAddressFamilyForm


@register_model_view(EIGRPAddressFamily, name='delete')
class EIGRPAddressFamilyDeleteView(ObjectDeleteView):
    queryset = EIGRPAddressFamily.objects.all()


@register_model_view(EIGRPAddressFamily, name='bulk_edit', detail=False)
class EIGRPAddressFamilyBulkEditView(BulkEditView):
    queryset = EIGRPAddressFamily.objects.all()
    table = EIGRPAddressFamilyTable
    filterset = EIGRPAddressFamilyFilterSet
    form = EIGRPAddressFamilyBulkEditForm


@register_model_view(EIGRPAddressFamily, name='bulk_delete', detail=False)
class EIGRPAddressFamilyBulkDeleteView(BulkDeleteView):
    queryset = EIGRPAddressFamily.objects.all()
    table = EIGRPAddressFamilyTable
    filterset = EIGRPAddressFamilyFilterSet


@register_model_view(EIGRPAddressFamily, name='bulk_import', detail=False)
class EIGRPAddressFamilyImportView(BulkImportView):
    queryset = EIGRPAddressFamily.objects.all()
    model_form = EIGRPAddressFamilyImportForm


#
# Network
#
@register_model_view(EIGRPNetwork, name='list', path='', detail=False)
class EIGRPNetworkListView(ObjectListView):
    queryset = EIGRPNetwork.objects.all()
    table = EIGRPNetworkTable
    filterset = EIGRPNetworkFilterSet
    filterset_form = EIGRPNetworkFilterForm


@register_model_view(EIGRPNetwork)
class EIGRPNetworkView(ObjectView):
    queryset = EIGRPNetwork.objects.all()
    template_name = 'netbox_routing/eigrpnetwork.html'


@register_model_view(EIGRPNetwork, name='add', detail=False)
@register_model_view(EIGRPNetwork, name='edit')
class EIGRPNetworkEditView(ObjectEditView):
    queryset = EIGRPNetwork.objects.all()
    form = EIGRPNetworkForm


@register_model_view(EIGRPNetwork, name='delete')
class EIGRPNetworkDeleteView(ObjectDeleteView):
    queryset = EIGRPNetwork.objects.all()


@register_model_view(EIGRPNetwork, name='bulk_edit', detail=False)
class EIGRPNetworkBulkEditView(BulkEditView):
    queryset = EIGRPNetwork.objects.all()
    filterset = EIGRPNetworkFilterSet
    table = EIGRPNetworkTable
    form = EIGRPNetworkBulkEditForm


@register_model_view(EIGRPNetwork, name='bulk_delete', detail=False)
class EIGRPNetworkBulkDeleteView(BulkDeleteView):
    queryset = EIGRPNetwork.objects.all()
    filterset = EIGRPNetworkFilterSet
    table = EIGRPNetworkTable


@register_model_view(EIGRPNetwork, name='bulk_import', detail=False)
class EIGRPNetworkImportView(BulkImportView):
    queryset = EIGRPNetwork.objects.all()
    model_form = EIGRPNetworkImportForm


#
# Interface
#
@register_model_view(EIGRPInterface, name='list', path='', detail=False)
class EIGRPInterfaceListView(ObjectListView):
    queryset = EIGRPInterface.objects.all()
    table = EIGRPInterfaceTable
    filterset = EIGRPInterfaceFilterSet
    filterset_form = EIGRPInterfaceFilterForm


@register_model_view(EIGRPInterface)
class EIGRPInterfaceView(ObjectView):
    queryset = EIGRPInterface.objects.all()
    template_name = 'netbox_routing/eigrpinterface.html'


@register_model_view(EIGRPInterface, name='add', detail=False)
@register_model_view(EIGRPInterface, name='edit')
class EIGRPInterfaceEditView(ObjectEditView):
    queryset = EIGRPInterface.objects.all()
    form = EIGRPInterfaceForm


@register_model_view(EIGRPInterface, name='delete')
class EIGRPInterfaceDeleteView(ObjectDeleteView):
    queryset = EIGRPInterface.objects.all()


@register_model_view(EIGRPInterface, name='bulk_edit', detail=False)
class EIGRPInterfaceBulkEditView(BulkEditView):
    queryset = EIGRPInterface.objects.all()
    filterset = EIGRPInterfaceFilterSet
    table = EIGRPInterfaceTable
    form = EIGRPInterfaceBulkEditForm


@register_model_view(EIGRPInterface, name='bulk_delete', detail=False)
class EIGRPInterfaceBulkDeleteView(BulkDeleteView):
    queryset = EIGRPInterface.objects.all()
    filterset = EIGRPInterfaceFilterSet
    table = EIGRPInterfaceTable


@register_model_view(EIGRPInterface, name='bulk_import', detail=False)
class EIGRPInterfaceImportView(BulkImportView):
    queryset = EIGRPInterface.objects.all()
    model_form = EIGRPInterfaceImportForm



import netaddr
from django.core.exceptions import ValidationError
from django.db import models
from django.urls import reverse
from django.utils.translation import gettext as _

from netbox.models import PrimaryModel

from netbox_routing import choices
from netbox_routing.choices.ospf import OSPFAreaTypeChoices
from netbox_routing.fields.ip import IPAddressField

__all__ = (
    'OSPFInstance',
    'OSPFArea',
    'OSPFInterface',
)


class OSPFInstance(PrimaryModel):
    name = models.CharField(max_length=100)
    router_id = IPAddressField(verbose_name=_('Router ID'))
    process_id = models.IntegerField(verbose_name=_('Process ID'))
    device = models.ForeignKey(
        to='dcim.Device',
        related_name='ospf_instances',
        on_delete=models.CASCADE,
        blank=False,
        null=False,
    )
    vrf = models.ForeignKey(
        verbose_name=_('VRF'),
        to='ipam.VRF',
        related_name='ospf_instances',
        on_delete=models.CASCADE,
        blank=True,
        null=True,
    )

    clone_fields = ('name', 'router_id', 'process_id', 'device', 'vrf')
    prerequisite_models = ('dcim.Device',)

    class Meta:
        ordering = ['vrf', 'router_id', 'process_id']
        verbose_name = 'OSPF Instance'

    def __str__(self):
        return f'{self.name} ({self.router_id})'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:ospfinstance', args=[self.pk])


class OSPFArea(PrimaryModel):
    area_id = models.CharField(max_length=100, verbose_name='Area ID')
    area_type = models.CharField(
        verbose_name=_('Area Type'),
        choices=OSPFAreaTypeChoices,
        blank=False,
        null=False,
        default='standard',
    )
    prerequisite_models = ()
    clone_fields = ('area_id', 'area_type')

    class Meta:
        ordering = ['area_id']
        verbose_name = 'OSPF Area'

    def __str__(self):
        return f'{self.area_id}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:ospfarea', args=[self.pk])

    def clean(self):
        super().clean()
        area_id = self.area_id
        try:
            int(area_id)
        except ValueError:
            try:
                str(netaddr.IPAddress(area_id))
            except netaddr.core.AddrFormatError:
                raise ValidationError(
                    {
                        'area_id': [
                            'This field must be an integer or a valid net address'
                        ]
                    }
                )


class OSPFInterface(PrimaryModel):
    instance = models.ForeignKey(
        to='netbox_routing.OSPFInstance',
        related_name='interfaces',
        on_delete=models.PROTECT,
        blank=False,
        null=False,
    )
    area = models.ForeignKey(
        to='netbox_routing.OSPFArea',
        on_delete=models.CASCADE,
        related_name='interfaces',
        blank=False,
        null=False,
    )
    interface = models.OneToOneField(
        to='dcim.Interface',
        related_name='ospf_interfaces',
        on_delete=models.CASCADE,
        blank=False,
        null=False,
    )
    passive = models.BooleanField(verbose_name='Passive', blank=True, null=True)
    priority = models.IntegerField(blank=True, null=True)
    bfd = models.BooleanField(blank=True, null=True, verbose_name='BFD')
    authentication = models.CharField(
        max_length=50, choices=choices.AuthenticationChoices, blank=True, null=True
    )
    passphrase = models.CharField(max_length=200, blank=True, null=True)

    clone_fields = (
        'device',
        'instance',
        'area',
        'interface',
        'passive',
        'priority',
        'bfd',
        'authentication',
        'passphrase',
    )
    prerequisite_models = (
        'netbox_routing.OSPFInstance',
        'netbox_routing.OSPFArea',
        'dcim.Interface',
    )

    class Meta:
        verbose_name = 'OSPF Interface'
        ordering = ('instance', 'area', 'interface')  # Name may be non-unique

    def __str__(self):
        return f'{self.interface.name}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:ospfinterface', args=[self.pk])



# Generated by Django 5.2.5 on 2026-01-16 23:05

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0017_netbox_v040500'),
    ]

    operations = [
        migrations.DeleteModel(
            name='BGPPoliyTemplate',
        ),
        migrations.DeleteModel(
            name='BGPSessionTemplate',
        ),
        migrations.DeleteModel(
            name='BGPPeerTemplate',
        ),
        migrations.DeleteModel(
            name='BGPPeerAddressFamily',
        ),
        migrations.DeleteModel(
            name='BGPPeer',
        ),
        migrations.DeleteModel(
            name='BGPScope',
        ),
        migrations.DeleteModel(
            name='BGPAddressFamily',
        ),
        migrations.DeleteModel(
            name='BGPRouter',
        ),
        migrations.DeleteModel(
            name='BGPSetting',
        ),
    ]



from django import template
from django.urls import reverse

register = template.Library()


@register.simple_tag()
def get_entry_url(method, instance, return_url):
    url = (
        f'plugins:{instance._meta.app_label}:{instance._meta.model_name}entry_{method}'
    )

    if not return_url:
        return_url = instance.get_absolute_url()
    return reverse(
        url,
        query={f'{instance._meta.model_name}': instance.pk, 'return_url': return_url},
    )



import django_filters
import netaddr

from django.core.exceptions import ValidationError
from django.db.models import Q
from django.utils.translation import gettext as _

from dcim.models import Device, Interface
from ipam.models import Prefix
from utilities.filters import MultiValueCharFilter
from utilities.filtersets import register_filterset

from netbox.filtersets import NetBoxModelFilterSet
from netbox_routing.models import (
    EIGRPAddressFamily,
    EIGRPRouter,
    EIGRPNetwork,
    EIGRPInterface,
)

__all__ = (
    'EIGRPRouterFilterSet',
    'EIGRPAddressFamilyFilterSet',
    'EIGRPNetworkFilterSet',
    'EIGRPInterfaceFilterSet',
)


class RouterMixin:

    def filter_rid(self, queryset, name, value):
        try:
            return queryset.filter(**{f'{name}__in': value})
        except ValidationError:
            return queryset.none()


@register_filterset
class EIGRPRouterFilterSet(RouterMixin, NetBoxModelFilterSet):
    device_id = django_filters.ModelMultipleChoiceFilter(
        field_name='device',
        queryset=Device.objects.all(),
        label='Device (ID)',
    )
    device = django_filters.ModelMultipleChoiceFilter(
        field_name='device__name',
        queryset=Device.objects.all(),
        to_field_name='name',
        label='Device',
    )
    rid = MultiValueCharFilter(
        method='filter_rid',
        label=_('Router ID'),
    )

    class Meta:
        model = EIGRPRouter
        fields = ('device_id', 'device', 'mode', 'rid')

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q()
        qs_filter |= Q(name__icontains=value)
        qs_filter |= Q(device__name__icontains=value)
        qs_filter |= Q(router_id__icontains=value)

        return queryset.filter(qs_filter).distinct()


class EIGRPAddressFamilyFilterSet(RouterMixin, NetBoxModelFilterSet):
    router_id = django_filters.ModelMultipleChoiceFilter(
        field_name='router',
        queryset=EIGRPRouter.objects.all(),
        label='Router (ID)',
    )
    router = django_filters.ModelMultipleChoiceFilter(
        field_name='router__name',
        queryset=EIGRPRouter.objects.all(),
        to_field_name='name',
        label='Router (Name)',
    )
    device_id = django_filters.ModelMultipleChoiceFilter(
        field_name='router__device',
        queryset=Device.objects.all(),
        label='Device (ID)',
    )
    device = django_filters.ModelMultipleChoiceFilter(
        field_name='router__device__name',
        queryset=Device.objects.all(),
        to_field_name='name',
        label='Device',
    )
    rid = MultiValueCharFilter(
        method='filter_rid',
        label=_('Router ID'),
    )

    class Meta:
        model = EIGRPAddressFamily
        fields = (
            'router_id',
            'router',
            'device_id',
            'device',
            'rid',
            'family',
        )

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(Q(rid__icontains=value))
        qs_filter |= Q(eigrprouternamed__name__icontains=value)
        return queryset.filter(qs_filter).distinct()


class EIGRPNetworkFilterSet(NetBoxModelFilterSet):
    router_id = django_filters.ModelMultipleChoiceFilter(
        field_name='router',
        queryset=EIGRPRouter.objects.all(),
        label='Router (ID)',
    )
    router = django_filters.ModelMultipleChoiceFilter(
        field_name='router__name',
        queryset=EIGRPRouter.objects.all(),
        to_field_name='name',
        label='Router (Name)',
    )
    address_family_id = django_filters.ModelMultipleChoiceFilter(
        field_name='address_family',
        queryset=EIGRPAddressFamily.objects.all(),
        label='Address Family (ID)',
    )
    device_id = django_filters.ModelMultipleChoiceFilter(
        field_name='router__device',
        queryset=Device.objects.all(),
        label='Device (ID)',
    )
    device = django_filters.ModelMultipleChoiceFilter(
        field_name='router__device__name',
        queryset=Device.objects.all(),
        to_field_name='name',
        label='Device',
    )
    network_id = django_filters.ModelMultipleChoiceFilter(
        field_name='network',
        queryset=Prefix.objects.all(),
        label='Network (ID)',
    )
    network = django_filters.CharFilter(
        method='filter_prefix',
        label=_('Network'),
    )

    class Meta:
        model = EIGRPNetwork
        fields = (
            'router_id',
            'router',
            'address_family_id',
            'address_family',
            'device_id',
            'device',
            'network_id',
            'network',
        )

    def filter_prefix(self, queryset, name, value):
        if not value.strip():
            return queryset
        try:
            query = str(netaddr.IPNetwork(value).cidr)
            return queryset.filter(prefix=query)
        except (netaddr.AddrFormatError, ValueError):
            return queryset.none()

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(
            Q(eigrprouternamed__name__icontains=value)
            | Q(address_family__rid__icontains=value)
        )
        qs_filter |= Q(network__contains=value.strip())
        try:
            prefix = str(netaddr.IPNetwork(value.strip()).cidr)
            qs_filter |= Q(network__net_contains_or_equals=prefix)
        except (netaddr.AddrFormatError, ValueError):
            pass
        return queryset.filter(qs_filter).distinct()


class EIGRPInterfaceFilterSet(NetBoxModelFilterSet):
    router_id = django_filters.ModelMultipleChoiceFilter(
        field_name='router',
        queryset=EIGRPRouter.objects.all(),
        label='Router (ID)',
    )
    router = django_filters.ModelMultipleChoiceFilter(
        field_name='router__name',
        queryset=EIGRPRouter.objects.all(),
        to_field_name='name',
        label='Router (Name)',
    )
    address_family_id = django_filters.ModelMultipleChoiceFilter(
        field_name='address_family',
        queryset=EIGRPAddressFamily.objects.all(),
        label='Address Family (ID)',
    )
    device_id = django_filters.ModelMultipleChoiceFilter(
        field_name='interface__device',
        queryset=Device.objects.all(),
        label='Device (ID)',
    )
    device = django_filters.ModelMultipleChoiceFilter(
        field_name='interface__device__name',
        queryset=Device.objects.all(),
        to_field_name='name',
        label='Device',
    )
    interface_id = django_filters.ModelMultipleChoiceFilter(
        field_name='interface',
        queryset=Interface.objects.all(),
        label='Interface (ID)',
    )
    interface = django_filters.ModelMultipleChoiceFilter(
        field_name='interface__name',
        queryset=Interface.objects.all(),
        to_field_name='name',
        label='Interface',
    )

    class Meta:
        model = EIGRPInterface
        fields = (
            'router_id',
            'router',
            'address_family_id',
            'address_family',
            'device_id',
            'device',
            'interface_id',
            'interface',
            'bfd',
            'passive',
            'authentication',
            'passphrase',
        )

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            Q(router__name__icontains=value)
            | Q(router__device__name__icontains=value)
            | Q(interface__name__icontains=value)
            | Q(interface__label__icontains=value)
            | Q(interface__device__name__icontains=value)
        )
        return queryset.filter(qs_filter).distinct()



import django_filters
from django.db.models import Q
from django.utils.translation import gettext as _

from netbox.filtersets import NetBoxModelFilterSet
from dcim.models import Device
from ipam.models import ASN, VRF, IPAddress
from utilities.filtersets import register_filterset

from netbox_routing.choices.bgp import *
from netbox_routing.models.bgp import *

__all__ = (
    'BGPSettingFilterSet',
    'BGPRouterFilterSet',
    'BGPScopeFilterSet',
    'BGPAddressFamilyFilterSet',
    'BGPPeerFilterSet',
    'BGPPeerTemplateFilterSet',
    'BGPPolicyTemplateFilterSet',
    'BGPSessionTemplateFilterSet',
    'BGPPeerAddressFamilyFilterSet',
    'BFDProfileFilterSet',
)


@register_filterset
class BGPPeerTemplateFilterSet(NetBoxModelFilterSet):
    peer_id = django_filters.ModelMultipleChoiceFilter(
        field_name='peers',
        queryset=BGPScope.objects.all(),
        label=_('Peer (ID)'),
    )
    address_family_id = django_filters.ModelMultipleChoiceFilter(
        field_name='address_families',
        queryset=BGPPeerAddressFamily.objects.all(),
        label=_('Address Family (ID)'),
    )
    remote_as_id = django_filters.ModelMultipleChoiceFilter(
        field_name='remote_as',
        queryset=ASN.objects.all(),
        label=_('Remote AS (ID)'),
    )
    remote_as = django_filters.ModelMultipleChoiceFilter(
        field_name='remote_as__asn',
        queryset=ASN.objects.all(),
        to_field_name='remote_as',
        label=_('Remote AS (ASN)'),
    )

    class Meta:
        model = BGPPeerTemplate
        fields = (
            'peer_id',
            'address_family_id',
            'remote_as_id',
            'remote_as',
            'enabled',
        )

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(name__icontains=value)
        return queryset.filter(qs_filter).distinct()


@register_filterset
class BGPPolicyTemplateFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = BGPPolicyTemplate
        fields = ('name',)

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(name__icontains=value)
        return queryset.filter(qs_filter).distinct()


@register_filterset
class BGPSessionTemplateFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = BGPSessionTemplate
        fields = ('name',)

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(name__icontains=value)
        return queryset.filter(qs_filter).distinct()


@register_filterset
class BGPSettingFilterSet(NetBoxModelFilterSet):
    key = django_filters.MultipleChoiceFilter(
        choices=BGPSettingChoices, null_value=None, label=_('Setting Name')
    )

    class Meta:
        model = BGPSetting
        fields = ('key',)

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(key__icontains=value)
        return queryset.filter(qs_filter).distinct()


@register_filterset
class BGPRouterFilterSet(NetBoxModelFilterSet):
    # device_id = django_filters.ModelMultipleChoiceFilter(
    #    field_name='device',
    #    queryset=Device.objects.all(),
    #    label=_('Device (ID)'),
    # )
    # device = django_filters.ModelMultipleChoiceFilter(
    #    field_name='device__name',
    #    queryset=Device.objects.all(),
    #    to_field_name='name',
    #    label=_('Device'),
    # )
    asn_id = django_filters.ModelMultipleChoiceFilter(
        field_name='asn',
        queryset=ASN.objects.all(),
        label=_('AS Number (ID)'),
    )
    asn = django_filters.ModelMultipleChoiceFilter(
        field_name='asn__asn',
        queryset=ASN.objects.all(),
        to_field_name='asn',
        label=_('AS Number'),
    )

    class Meta:
        model = BGPRouter
        fields = ('asn_id', 'asn')

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            Q(name__icontains=value)
            | Q(device__name__icontains=value)
            | Q(asn__asn__icontains=value)
        )
        return queryset.filter(qs_filter).distinct()


@register_filterset
class BGPScopeFilterSet(NetBoxModelFilterSet):
    router_id = django_filters.ModelMultipleChoiceFilter(
        field_name='router',
        queryset=BGPRouter.objects.all(),
        label=_('Router (ID)'),
    )
    vrf_id = django_filters.ModelMultipleChoiceFilter(
        field_name='vrf',
        queryset=VRF.objects.all(),
        label=_('VRF (ID)'),
    )
    vrf = django_filters.ModelMultipleChoiceFilter(
        field_name='vrf__name',
        queryset=VRF.objects.all(),
        to_field_name='vrf',
        label=_('VRF'),
    )

    class Meta:
        model = BGPScope
        fields = ('router_id', 'vrf_id', 'vrf')

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            Q(router__device__name__icontains=value)
            | Q(router__asn__asn__icontains=value)
            | Q(vrf__name__icontains=value)
        )
        return queryset.filter(qs_filter).distinct()


@register_filterset
class BGPAddressFamilyFilterSet(NetBoxModelFilterSet):
    scope_id = django_filters.ModelMultipleChoiceFilter(
        field_name='scope',
        queryset=Device.objects.all(),
        label=_('Router (ID)'),
    )
    address_family = django_filters.MultipleChoiceFilter(
        choices=BGPAddressFamilyChoices, null_value=None, label=_('Address Family')
    )

    class Meta:
        model = BGPAddressFamily
        fields = ('scope_id', 'address_family')

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(address_family__icontains=value)
        return queryset.filter(qs_filter).distinct()


@register_filterset
class BGPPeerFilterSet(NetBoxModelFilterSet):
    scope_id = django_filters.ModelMultipleChoiceFilter(
        field_name='scope',
        queryset=BGPScope.objects.all(),
        label=_('Scope (ID)'),
    )
    peer_id = django_filters.ModelMultipleChoiceFilter(
        field_name='peer',
        queryset=IPAddress.objects.all(),
        label=_('Peer (ID)'),
    )
    peer = django_filters.ModelMultipleChoiceFilter(
        field_name='peer__address',
        queryset=IPAddress.objects.all(),
        to_field_name='peer',
        label=_('Peer (Name)'),
    )
    peer_group_id = django_filters.ModelMultipleChoiceFilter(
        field_name='peer_group',
        queryset=BGPPeerTemplate.objects.all(),
        label=_('Peer Group (ID)'),
    )
    remote_as_id = django_filters.ModelMultipleChoiceFilter(
        field_name='remote_as',
        queryset=ASN.objects.all(),
        label=_('Remote AS (ID)'),
    )
    remote_as = django_filters.ModelMultipleChoiceFilter(
        field_name='remote_as__asn',
        queryset=ASN.objects.all(),
        to_field_name='remote_as',
        label=_('Remote AS (ASN)'),
    )
    local_as_id = django_filters.ModelMultipleChoiceFilter(
        field_name='local_as',
        queryset=ASN.objects.all(),
        label=_('Local AS (ID)'),
    )
    local_as = django_filters.ModelMultipleChoiceFilter(
        field_name='local_as__asn',
        queryset=ASN.objects.all(),
        to_field_name='local_as',
        label=_('Local AS (ASN)'),
    )

    class Meta:
        model = BGPPeer
        fields = ('scope_id', 'peer_id', 'remote_as_id', 'local_as_id')

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(name__icontains=value)
        qs_filter |= Q(peer__address__icontains=value)
        qs_filter |= Q(remote_as__asn__icontains=value)
        qs_filter |= Q(local_as__asn__icontains=value)
        return queryset.filter(qs_filter).distinct()


@register_filterset
class BGPPeerAddressFamilyFilterSet(NetBoxModelFilterSet):
    peer_id = django_filters.ModelMultipleChoiceFilter(
        field_name='peer',
        queryset=BGPScope.objects.all(),
        label=_('Peer (ID)'),
    )
    peer_group_id = django_filters.ModelMultipleChoiceFilter(
        field_name='peer_group',
        queryset=BGPPeerTemplate.objects.all(),
        label=_('Peer Group (ID)'),
    )
    address_family_id = django_filters.ModelMultipleChoiceFilter(
        field_name='address_family',
        queryset=BGPAddressFamily.objects.all(),
        label=_('Address Family (ID)'),
    )

    class Meta:
        model = BGPPeerAddressFamily
        fields = (
            'peer_id',
            'peer_group_id',
            'address_family',
            'enabled',
        )

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(peer__address__icontains=value)
        qs_filter |= Q(remote_as__asn__icontains=value)
        qs_filter |= Q(local_as__asn__icontains=value)
        return queryset.filter(qs_filter).distinct()


@register_filterset
class BFDProfileFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = BFDProfile
        fields = ('name',)

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(name__icontains=value)
        return queryset.filter(qs_filter).distinct()



from .bgp import *
from .community import *
from .eigrp import EIGRPRouterIndex, EIGRPAddressFamilyIndex
from .objects import *
from .ospf import OSPFInstanceIndex, OSPFAreaIndex
from .static import StaticRouteIndex

__all__ = (
    'ASPathIndex',
    'ASPathEntryIndex',
    'BGPRouterIndex',
    'BGPScopeIndex',
    'BGPAddressFamilyIndex',
    'BGPPeerIndex',
    'BGPPeerAddressFamilyIndex',
    'BGPPeerTemplateIndex',
    'BGPPolicyTemplateIndex',
    'BGPSessionTemplateIndex',
    'CommunityIndex',
    'CommunityListIndex',
    'CommunityListEntryIndex',
    'EIGRPRouterIndex',
    'EIGRPAddressFamilyIndex',
    'OSPFInstanceIndex',
    'OSPFAreaIndex',
    'PrefixListIndex',
    'PrefixListEntryIndex',
    'RouteMapIndex',
    'RouteMapEntryIndex',
    'StaticRouteIndex',
)



from typing import Annotated

import strawberry
import strawberry_django
from strawberry import ID

from ipam.graphql.filters import RoleFilter
from netbox.graphql.filters import PrimaryModelFilter
from netbox_routing import models
from tenancy.graphql.filter_mixins import TenancyFilterMixin

__all__ = (
    'CommunityFilter',
    'CommunityListFilter',
    'CommunityListEntryFilter',
)


@strawberry_django.filter(models.Community, lookups=True)
class CommunityFilter(TenancyFilterMixin, PrimaryModelFilter):
    role: (
        Annotated[
            'RoleFilter',
            strawberry.lazy('ipam.graphql.filters'),
        ]
        | None
    ) = strawberry_django.filter_field()
    role_id: ID | None = strawberry_django.filter_field()


@strawberry_django.filter(models.CommunityList, lookups=True)
class CommunityListFilter(TenancyFilterMixin, PrimaryModelFilter):
    pass


@strawberry_django.filter(models.CommunityListEntry, lookups=True)
class CommunityListEntryFilter(PrimaryModelFilter):
    community_list: (
        Annotated[
            'CommunityListFilter',
            strawberry.lazy('netbox_routing.graphql.filters'),
        ]
        | None
    ) = strawberry_django.filter_field()
    community_list_id: ID | None = strawberry_django.filter_field()

    community: (
        Annotated[
            'CommunityFilter',
            strawberry.lazy('netbox_routing.graphql.filters'),
        ]
        | None
    ) = strawberry_django.filter_field()
    community_id: ID | None = strawberry_django.filter_field()



import django_filters
import netaddr
from django.db.models import Q
from django.utils.translation import gettext as _

from ipam.models import Prefix
from netbox.filtersets import NetBoxModelFilterSet
from utilities.filtersets import register_filterset

from netbox_routing.models.objects import *
from netbox_routing.models.community import *

__all__ = (
    'CustomPrefixFilterSet',
    'PrefixListFilterSet',
    'PrefixListEntryFilterSet',
    'RouteMapFilterSet',
    'RouteMapEntryFilterSet',
    'ASPathFilterSet',
    'ASPathEntryFilterSet',
)


class ASPathFilterSet(NetBoxModelFilterSet):
    route_map_entry_id = django_filters.ModelMultipleChoiceFilter(
        field_name='route_map_entries',
        queryset=RouteMapEntry.objects.all(),
        label=_('Route Map Entry (ID)'),
    )

    class Meta:
        model = ASPath
        fields = ()

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(name__icontains=value)
        return queryset.filter(qs_filter).distinct()


@register_filterset
class ASPathEntryFilterSet(NetBoxModelFilterSet):
    aspath_id = django_filters.ModelMultipleChoiceFilter(
        field_name='aspath',
        queryset=ASPath.objects.all(),
        label=_('AS-Path (ID)'),
    )
    aspath = django_filters.ModelMultipleChoiceFilter(
        field_name='aspath__name',
        queryset=ASPath.objects.all(),
        to_field_name='name',
        label=_('AS-Path (Name)'),
    )

    class Meta:
        model = ASPathEntry
        fields = (
            'aspath_id',
            'aspath',
            'action',
        )

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            Q(aspath__name__icontains=value)
            | Q(pattern__icontains=value)
            | Q(action=value)
        )
        return queryset.filter(qs_filter).distinct()


@register_filterset
class PrefixListFilterSet(NetBoxModelFilterSet):
    route_map_entry_id = django_filters.ModelMultipleChoiceFilter(
        field_name='route_map_entries',
        queryset=RouteMapEntry.objects.all(),
        label=_('Route Map Entry (ID)'),
    )

    class Meta:
        model = PrefixList
        fields = ('family',)

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(name__icontains=value)
        return queryset.filter(qs_filter).distinct()


class PrefixListEntryFilterSet(NetBoxModelFilterSet):
    prefix_list_id = django_filters.ModelMultipleChoiceFilter(
        field_name='prefix_list',
        queryset=PrefixList.objects.all(),
        label=_('Prefix List (ID)'),
    )
    prefix_list = django_filters.ModelMultipleChoiceFilter(
        field_name='prefix_list__name',
        queryset=PrefixList.objects.all(),
        to_field_name='name',
        label=_('Prefix List (Name)'),
    )
    prefix_id = django_filters.ModelMultipleChoiceFilter(
        field_name='prefixes',
        queryset=Prefix.objects.all(),
        label=_('Prefix (ID)'),
    )
    prefix = django_filters.ModelMultipleChoiceFilter(
        field_name='prefixes__prefix',
        queryset=Prefix.objects.all(),
        to_field_name='prefix',
        label=_('Prefix (Name)'),
    )
    custom_prefix_id = django_filters.ModelMultipleChoiceFilter(
        field_name='custom_prefixes',
        queryset=CustomPrefix.objects.all(),
        label=_('Custom Prefix (ID)'),
    )
    custom_prefix = django_filters.ModelMultipleChoiceFilter(
        field_name='custom_prefixes__prefix',
        queryset=CustomPrefix.objects.all(),
        to_field_name='prefix',
        label=_('Custom Prefix (Name)'),
    )

    class Meta:
        model = PrefixListEntry
        fields = (
            'prefix_list_id',
            'prefix_list',
            'prefix_id',
            'prefix',
            'custom_prefix_id',
            'custom_prefix',
            'sequence',
            'action',
            'le',
            'ge',
        )

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(prefix_list__name__icontains=value)
        qs_filter |= Q(prefixes__prefix__icontains=value)
        qs_filter |= Q(custom_prefixes__prefix__icontains=value)

        return queryset.filter(qs_filter).distinct()

    def filter_prefix(self, queryset, name, value):
        if not value.strip():
            return queryset
        try:
            query = str(netaddr.IPNetwork(value).cidr)
            return queryset.filter(prefix=query)
        except (netaddr.AddrFormatError, ValueError):
            return queryset.none()


class CustomPrefixFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = CustomPrefix
        fields = ()

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(prefix__icontains=value)

        return queryset.filter(qs_filter).distinct()


@register_filterset
class RouteMapFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = RouteMap
        fields = ()

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(name__icontains=value)
        return queryset.filter(qs_filter).distinct()


@register_filterset
class RouteMapEntryFilterSet(NetBoxModelFilterSet):
    route_map_id = django_filters.ModelMultipleChoiceFilter(
        field_name='route_map',
        queryset=RouteMap.objects.all(),
        label=_('Route Map (ID)'),
    )
    route_map = django_filters.ModelMultipleChoiceFilter(
        field_name='route_map__name',
        queryset=RouteMap.objects.all(),
        to_field_name='name',
        label=_('Route Map (Name)'),
    )
    match_prefix_list_id = django_filters.ModelMultipleChoiceFilter(
        field_name='match_prefix_list',
        queryset=PrefixList.objects.all(),
        label=_('Prefix List (ID)'),
    )
    match_prefix_list = django_filters.ModelMultipleChoiceFilter(
        field_name='match_prefix_list__name',
        queryset=PrefixList.objects.all(),
        to_field_name='name',
        label=_('Prefix List (Name)'),
    )
    match_community_list_id = django_filters.ModelMultipleChoiceFilter(
        field_name='match_community_list',
        queryset=CommunityList.objects.all(),
        label=_('Community List (ID)'),
    )
    match_community_list = django_filters.ModelMultipleChoiceFilter(
        field_name='match_community_list__name',
        queryset=CommunityList.objects.all(),
        to_field_name='name',
        label=_('Community List (Name)'),
    )
    match_community_id = django_filters.ModelMultipleChoiceFilter(
        field_name='match_community',
        queryset=Community.objects.all(),
        label=_('Community (ID)'),
    )
    match_community = django_filters.ModelMultipleChoiceFilter(
        field_name='match_community__community',
        queryset=Community.objects.all(),
        to_field_name='community',
        label=_('Community (Community)'),
    )
    match_aspath_id = django_filters.ModelMultipleChoiceFilter(
        field_name='match_aspath',
        queryset=ASPath.objects.all(),
        label=_('AS Path List (ID)'),
    )
    match_aspath = django_filters.ModelMultipleChoiceFilter(
        field_name='match_aspath__name',
        queryset=ASPath.objects.all(),
        to_field_name='name',
        label=_('AS Path List (Name)'),
    )

    class Meta:
        model = RouteMapEntry
        fields = (
            'route_map_id',
            'route_map',
            'sequence',
            'action',
            'match_prefix_list_id',
            'match_prefix_list',
            'match_community_list_id',
            'match_community_list',
            'match_community_id',
            'match_community',
            'match_aspath_id',
            'match_aspath',
        )

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(route_map__name__icontains=value)
        qs_filter |= Q(match__icontains=value)
        qs_filter |= Q(set__icontains=value)
        return queryset.filter(qs_filter).distinct()



from django.db.models import Q

__all__ = ('PREFIX_ASSIGNMENT_MODELS',)

PREFIX_ASSIGNMENT_MODELS = Q(
    Q(app_label='ipam', model='prefix')
    | Q(app_label='netbox_routing', model='customprefix')
)



from .bgp.filters import *
from .community.filters import *
from .eigrp.filters import *
from .objects.filters import *
from .ospf.filters import *
from .static.filters import *

__all__ = (
    'ASPathFilter',
    'ASPathEntryFilter',
    'BGPSettingFilter',
    'BGPPeerTemplateFilter',
    'BGPPolicyTemplateFilter',
    'BGPSessionTemplateFilter',
    'BGPRouterFilter',
    'BGPScopeFilter',
    'BGPAddressFamilyFilter',
    'BGPPeerFilter',
    'BGPPeerAddressFamilyFilter',
    'CommunityFilter',
    'CommunityListFilter',
    'CommunityListEntryFilter',
    'EIGRPRouterFilter',
    'EIGRPAddressFamilyFilter',
    'EIGRPNetworkFilter',
    'EIGRPInterfaceFilter',
    'OSPFInstanceFilter',
    'OSPFAreaFilter',
    'OSPFInterfaceFilter',
    'PrefixListFilter',
    'PrefixListEntryFilter',
    'RouteMapFilter',
    'RouteMapEntryFilter',
    'StaticRouteFilter',
)
